#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: dynamodb_table
version_added: 1.0.0
short_description: Create, update or delete AWS Dynamo DB tables
description:
  - Create or delete AWS Dynamo DB tables.
  - Can update the provisioned throughput on existing tables.
  - Returns the status of the specified table.
author:
  - Alan Loi (@loia)
options:
  state:
    description:
      - Create or delete the table.
    choices: ['present', 'absent']
    default: 'present'
    type: str
  name:
    description:
      - Name of the table.
    required: true
    type: str
  hash_key_name:
    description:
      - Name of the hash key.
      - Required when I(state=present) and table doesn't exist.
    type: str
  hash_key_type:
    description:
      - Type of the hash key.
      - Defaults to C('STRING') when creating a new table.
    choices: ['STRING', 'NUMBER', 'BINARY']
    type: str
  range_key_name:
    description:
      - Name of the range key.
    type: str
  range_key_type:
    description:
      - Type of the range key.
      - Defaults to C('STRING') when creating a new range key.
    choices: ['STRING', 'NUMBER', 'BINARY']
    type: str
  billing_mode:
    description:
      - Controls whether provisoned pr on-demand tables are created.
    choices: ['PROVISIONED', 'PAY_PER_REQUEST']
    type: str
  read_capacity:
    description:
      - Read throughput capacity (units) to provision.
      - Defaults to C(1) when creating a new table.
    type: int
  write_capacity:
    description:
      - Write throughput capacity (units) to provision.
      - Defaults to C(1) when creating a new table.
    type: int
  indexes:
    description:
      - list of dictionaries describing indexes to add to the table. global indexes can be updated. local indexes don't support updates or have throughput.
      - "required options: ['name', 'type', 'hash_key_name']"
      - "other options: ['hash_key_type', 'range_key_name', 'range_key_type', 'includes', 'read_capacity', 'write_capacity']"
    suboptions:
      name:
        description: The name of the index.
        type: str
        required: true
      type:
        description:
          - The type of index.
        type: str
        required: true
        choices: ['all', 'global_all', 'global_include', 'global_keys_only', 'include', 'keys_only']
      hash_key_name:
        description:
          - The name of the hash-based key.
          - Required if index doesn't already exist.
          - Can not be modified once the index has been created.
        required: false
        type: str
      hash_key_type:
        description:
          - The type of the hash-based key.
          - Defaults to C('STRING') when creating a new index.
          - Can not be modified once the index has been created.
        type: str
        choices: ['STRING', 'NUMBER', 'BINARY']
      range_key_name:
        description:
          - The name of the range-based key.
          - Can not be modified once the index has been created.
        type: str
      range_key_type:
        type: str
        description:
          - The type of the range-based key.
          - Defaults to C('STRING') when creating a new index.
          - Can not be modified once the index has been created.
        choices: ['STRING', 'NUMBER', 'BINARY']
      includes:
        type: list
        description: A list of fields to include when using C(global_include) or C(include) indexes.
        elements: str
      read_capacity:
        description:
          - Read throughput capacity (units) to provision for the index.
        type: int
      write_capacity:
        description:
          - Write throughput capacity (units) to provision for the index.
        type: int
    default: []
    type: list
    elements: dict
  table_class:
    description:
      - The class of the table.
    choices: ['STANDARD', 'STANDARD_INFREQUENT_ACCESS']
    type: str
    version_added: 3.1.0
  wait_timeout:
    description:
      - How long (in seconds) to wait for creation / update / deletion to complete.
      - AWS only allows secondary indexies to be updated one at a time, this module will automatically update them
        in serial, and the timeout will be separately applied for each index.
    aliases: ['wait_for_active_timeout']
    default: 900
    type: int
  wait:
    description:
      - When I(wait=True) the module will wait for up to I(wait_timeout) seconds
        for index updates, table creation or deletion to complete before returning.
    default: True
    type: bool
extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.tags
  - amazon.aws.boto3
"""

EXAMPLES = r"""
- name: Create dynamo table with hash and range primary key
  community.aws.dynamodb_table:
    name: my-table
    region: us-east-1
    hash_key_name: id
    hash_key_type: STRING
    range_key_name: create_time
    range_key_type: NUMBER
    read_capacity: 2
    write_capacity: 2
    tags:
      tag_name: tag_value

- name: Update capacity on existing dynamo table
  community.aws.dynamodb_table:
    name: my-table
    region: us-east-1
    read_capacity: 10
    write_capacity: 10

- name: Create pay-per-request table
  community.aws.dynamodb_table:
    name: my-table
    region: us-east-1
    hash_key_name: id
    hash_key_type: STRING
    billing_mode: PAY_PER_REQUEST

- name: set index on existing dynamo table
  community.aws.dynamodb_table:
    name: my-table
    region: us-east-1
    indexes:
      - name: NamedIndex
        type: global_include
        hash_key_name: id
        range_key_name: create_time
        includes:
          - other_field
          - other_field2
        read_capacity: 10
        write_capacity: 10

- name: Delete dynamo table
  community.aws.dynamodb_table:
    name: my-table
    region: us-east-1
    state: absent
"""

RETURN = r"""
table:
  description: The returned table params from the describe API call.
  returned: success
  type: complex
  contains: {}
  sample: {
    "arn": "arn:aws:dynamodb:us-east-1:721066863947:table/ansible-test-table",
    "attribute_definitions": [
        {
            "attribute_name": "id",
            "attribute_type": "N"
        }
    ],
    "billing_mode": "PROVISIONED",
    "creation_date_time": "2022-02-04T13:36:01.578000+00:00",
    "id": "533b45fe-0870-4b66-9b00-d2afcfe96f19",
    "item_count": 0,
    "key_schema": [
        {
            "attribute_name": "id",
            "key_type": "HASH"
        }
    ],
    "name": "ansible-test-14482047-alinas-mbp",
    "provisioned_throughput": {
        "number_of_decreases_today": 0,
        "read_capacity_units": 1,
        "write_capacity_units": 1
    },
    "size": 0,
    "status": "ACTIVE",
    "table_arn": "arn:aws:dynamodb:us-east-1:721066863947:table/ansible-test-table",
    "table_id": "533b45fe-0870-4b66-9b00-d2afcfe96f19",
    "table_name": "ansible-test-table",
    "table_size_bytes": 0,
    "table_status": "ACTIVE",
    "tags": {}
  }
table_status:
  description: The current status of the table.
  returned: success
  type: str
  sample: ACTIVE
"""

try:
    import botocore
except ImportError:
    pass  # Handled by AnsibleAWSModule

from ansible.module_utils.common.dict_transformations import camel_dict_to_snake_dict

from ansible_collections.amazon.aws.plugins.module_utils.botocore import is_boto3_error_code
from ansible_collections.amazon.aws.plugins.module_utils.retries import AWSRetry
from ansible_collections.amazon.aws.plugins.module_utils.tagging import ansible_dict_to_boto3_tag_list
from ansible_collections.amazon.aws.plugins.module_utils.tagging import boto3_tag_list_to_ansible_dict
from ansible_collections.amazon.aws.plugins.module_utils.tagging import compare_aws_tags

from ansible_collections.community.aws.plugins.module_utils.dynamodb import wait_indexes_active
from ansible_collections.community.aws.plugins.module_utils.dynamodb import wait_table_exists
from ansible_collections.community.aws.plugins.module_utils.dynamodb import wait_table_not_exists
from ansible_collections.community.aws.plugins.module_utils.modules import AnsibleCommunityAWSModule as AnsibleAWSModule

DYNAMO_TYPE_DEFAULT = "STRING"
INDEX_REQUIRED_OPTIONS = ["name", "type", "hash_key_name"]
INDEX_OPTIONS = INDEX_REQUIRED_OPTIONS + [
    "hash_key_type",
    "range_key_name",
    "range_key_type",
    "includes",
    "read_capacity",
    "write_capacity",
]
INDEX_TYPE_OPTIONS = ["all", "global_all", "global_include", "global_keys_only", "include", "keys_only"]
# Map in both directions
DYNAMO_TYPE_MAP_LONG = {"STRING": "S", "NUMBER": "N", "BINARY": "B"}
DYNAMO_TYPE_MAP_SHORT = dict((v, k) for k, v in DYNAMO_TYPE_MAP_LONG.items())
KEY_TYPE_CHOICES = list(DYNAMO_TYPE_MAP_LONG.keys())


# If you try to update an index while another index is updating, it throws
# LimitExceededException/ResourceInUseException exceptions at you.  This can be
# pretty slow, so add plenty of retries...
@AWSRetry.jittered_backoff(
    retries=45,
    delay=5,
    max_delay=30,
    catch_extra_error_codes=["ResourceInUseException", "ResourceNotFoundException"],
)
def _update_table_with_long_retry(**changes):
    return client.update_table(TableName=module.params.get("name"), **changes)


# ResourceNotFoundException is expected here if the table doesn't exist
@AWSRetry.jittered_backoff(catch_extra_error_codes=["LimitExceededException", "ResourceInUseException"])
def _describe_table(**params):
    return client.describe_table(**params)


def wait_exists():
    wait_table_exists(
        module,
        module.params.get("wait_timeout"),
        module.params.get("name"),
    )


def wait_not_exists():
    wait_table_not_exists(
        module,
        module.params.get("wait_timeout"),
        module.params.get("name"),
    )


def wait_indexes():
    wait_indexes_active(
        module,
        module.params.get("wait_timeout"),
        module.params.get("name"),
    )


def _short_type_to_long(short_key):
    if not short_key:
        return None
    return DYNAMO_TYPE_MAP_SHORT.get(short_key, None)


def _long_type_to_short(long_key):
    if not long_key:
        return None
    return DYNAMO_TYPE_MAP_LONG.get(long_key, None)


def _schema_dict(key_name, key_type):
    return dict(
        AttributeName=key_name,
        KeyType=key_type,
    )


def _merge_index_params(index, current_index):
    idx = dict(current_index)
    idx.update(index)
    return idx


def _decode_primary_index(current_table):
    """
    Decodes the primary index info from the current table definition
    splitting it up into the keys we use as parameters
    """
    # The schema/attribute definitions are a list of dicts which need the same
    # treatment as boto3's tag lists
    schema = boto3_tag_list_to_ansible_dict(
        current_table.get("key_schema", []),
        # Map from 'HASH'/'RANGE' to attribute name
        tag_name_key_name="key_type",
        tag_value_key_name="attribute_name",
    )
    attributes = boto3_tag_list_to_ansible_dict(
        current_table.get("attribute_definitions", []),
        # Map from attribute name to 'S'/'N'/'B'.
        tag_name_key_name="attribute_name",
        tag_value_key_name="attribute_type",
    )

    hash_key_name = schema.get("HASH")
    hash_key_type = _short_type_to_long(attributes.get(hash_key_name, None))
    range_key_name = schema.get("RANGE", None)
    range_key_type = _short_type_to_long(attributes.get(range_key_name, None))

    return dict(
        hash_key_name=hash_key_name,
        hash_key_type=hash_key_type,
        range_key_name=range_key_name,
        range_key_type=range_key_type,
    )


def _decode_index(index_data, attributes, type_prefix=""):
    try:
        index_map = dict(
            name=index_data["index_name"],
        )

        index_data = dict(index_data)
        index_data["attribute_definitions"] = attributes

        index_map.update(_decode_primary_index(index_data))

        throughput = index_data.get("provisioned_throughput", {})
        index_map["provisioned_throughput"] = throughput
        if throughput:
            index_map["read_capacity"] = throughput.get("read_capacity_units")
            index_map["write_capacity"] = throughput.get("write_capacity_units")

        projection = index_data.get("projection", {})
        if projection:
            index_map["type"] = type_prefix + projection.get("projection_type")
            index_map["includes"] = projection.get("non_key_attributes", [])

        return index_map
    except Exception as e:
        module.fail_json_aws(e, msg="Decode failure", index_data=index_data)


def compatability_results(current_table):
    if not current_table:
        return dict()

    billing_mode = current_table.get("billing_mode")

    primary_indexes = _decode_primary_index(current_table)

    hash_key_name = primary_indexes.get("hash_key_name")
    hash_key_type = primary_indexes.get("hash_key_type")
    range_key_name = primary_indexes.get("range_key_name")
    range_key_type = primary_indexes.get("range_key_type")

    indexes = list()
    global_indexes = current_table.get("_global_index_map", {})
    local_indexes = current_table.get("_local_index_map", {})
    for index in global_indexes:
        idx = dict(global_indexes[index])
        idx.pop("provisioned_throughput", None)
        indexes.append(idx)
    for index in local_indexes:
        idx = dict(local_indexes[index])
        idx.pop("provisioned_throughput", None)
        indexes.append(idx)

    compat_results = dict(
        hash_key_name=hash_key_name,
        hash_key_type=hash_key_type,
        range_key_name=range_key_name,
        range_key_type=range_key_type,
        indexes=indexes,
        billing_mode=billing_mode,
        region=module.region,
        table_name=current_table.get("table_name", None),
        table_class=current_table.get("table_class_summary", {}).get("table_class", None),
        table_status=current_table.get("table_status", None),
        tags=current_table.get("tags", {}),
    )

    if billing_mode == "PROVISIONED":
        throughput = current_table.get("provisioned_throughput", {})
        compat_results["read_capacity"] = throughput.get("read_capacity_units", None)
        compat_results["write_capacity"] = throughput.get("write_capacity_units", None)

    return compat_results


def get_dynamodb_table():
    table_name = module.params.get("name")
    try:
        table = _describe_table(TableName=table_name)
    except is_boto3_error_code("ResourceNotFoundException"):
        return None
    except (
        botocore.exceptions.ClientError,
        botocore.exceptions.BotoCoreError,
    ) as e:  # pylint: disable=duplicate-except
        module.fail_json_aws(e, msg="Failed to describe table")

    table = table["Table"]
    try:
        tags = client.list_tags_of_resource(aws_retry=True, ResourceArn=table["TableArn"])["Tags"]
    except is_boto3_error_code("AccessDeniedException"):
        module.warn("Permission denied when listing tags")
        tags = []
    except (
        botocore.exceptions.ClientError,
        botocore.exceptions.BotoCoreError,
    ) as e:  # pylint: disable=duplicate-except
        module.fail_json_aws(e, msg="Failed to list table tags")

    tags = boto3_tag_list_to_ansible_dict(tags)

    table = camel_dict_to_snake_dict(table)

    # Put some of the values into places people will expect them
    table["arn"] = table["table_arn"]
    table["name"] = table["table_name"]
    table["status"] = table["table_status"]
    table["id"] = table["table_id"]
    table["size"] = table["table_size_bytes"]
    table["tags"] = tags

    if "table_class_summary" in table:
        table["table_class"] = table["table_class_summary"]["table_class"]

    # billing_mode_summary doesn't always seem to be set but is always set for PAY_PER_REQUEST
    # and when updating the billing_mode
    if "billing_mode_summary" in table:
        table["billing_mode"] = table["billing_mode_summary"]["billing_mode"]
    else:
        table["billing_mode"] = "PROVISIONED"

    # convert indexes into something we can easily search against
    attributes = table["attribute_definitions"]
    global_index_map = dict()
    local_index_map = dict()
    for index in table.get("global_secondary_indexes", []):
        idx = _decode_index(index, attributes, type_prefix="global_")
        global_index_map[idx["name"]] = idx
    for index in table.get("local_secondary_indexes", []):
        idx = _decode_index(index, attributes)
        local_index_map[idx["name"]] = idx
    table["_global_index_map"] = global_index_map
    table["_local_index_map"] = local_index_map

    return table


def _generate_attribute_map():
    """
    Builds a map of Key Names to Type
    """
    attributes = dict()

    for index in (module.params, *module.params.get("indexes")):
        # run through hash_key_name and range_key_name
        for t in ["hash", "range"]:
            key_name = index.get(t + "_key_name")
            if not key_name:
                continue
            key_type = index.get(t + "_key_type") or DYNAMO_TYPE_DEFAULT
            _type = _long_type_to_short(key_type)
            if key_name in attributes:
                if _type != attributes[key_name]:
                    module.fail_json(
                        msg="Conflicting attribute type", type_1=_type, type_2=attributes[key_name], key_name=key_name
                    )
            else:
                attributes[key_name] = _type

    return attributes


def _generate_attributes():
    attributes = _generate_attribute_map()

    # Use ansible_dict_to_boto3_tag_list to generate the list of dicts
    # format we need
    attrs = ansible_dict_to_boto3_tag_list(
        attributes, tag_name_key_name="AttributeName", tag_value_key_name="AttributeType"
    )
    return list(attrs)


def _generate_throughput(params=None):
    if not params:
        params = module.params

    read_capacity = params.get("read_capacity") or 1
    write_capacity = params.get("write_capacity") or 1
    throughput = dict(
        ReadCapacityUnits=read_capacity,
        WriteCapacityUnits=write_capacity,
    )

    return throughput


def _generate_schema(params=None):
    if not params:
        params = module.params

    schema = list()
    hash_key_name = params.get("hash_key_name")
    range_key_name = params.get("range_key_name")

    if hash_key_name:
        entry = _schema_dict(hash_key_name, "HASH")
        schema.append(entry)
    if range_key_name:
        entry = _schema_dict(range_key_name, "RANGE")
        schema.append(entry)

    return schema


def _primary_index_changes(current_table):
    primary_index = _decode_primary_index(current_table)

    hash_key_name = primary_index.get("hash_key_name")
    _hash_key_name = module.params.get("hash_key_name")
    hash_key_type = primary_index.get("hash_key_type")
    _hash_key_type = module.params.get("hash_key_type")
    range_key_name = primary_index.get("range_key_name")
    _range_key_name = module.params.get("range_key_name")
    range_key_type = primary_index.get("range_key_type")
    _range_key_type = module.params.get("range_key_type")

    changed = list()

    if _hash_key_name and (_hash_key_name != hash_key_name):
        changed.append("hash_key_name")
    if _hash_key_type and (_hash_key_type != hash_key_type):
        changed.append("hash_key_type")
    if _range_key_name and (_range_key_name != range_key_name):
        changed.append("range_key_name")
    if _range_key_type and (_range_key_type != range_key_type):
        changed.append("range_key_type")

    return changed


def _throughput_changes(current_table, params=None):
    if not params:
        params = module.params

    throughput = current_table.get("provisioned_throughput", {})
    read_capacity = throughput.get("read_capacity_units", None)
    _read_capacity = params.get("read_capacity") or read_capacity
    write_capacity = throughput.get("write_capacity_units", None)
    _write_capacity = params.get("write_capacity") or write_capacity

    if (read_capacity != _read_capacity) or (write_capacity != _write_capacity):
        return dict(
            ReadCapacityUnits=_read_capacity,
            WriteCapacityUnits=_write_capacity,
        )

    return dict()


def _generate_global_indexes(billing_mode):
    index_exists = dict()
    indexes = list()

    include_throughput = True

    if billing_mode == "PAY_PER_REQUEST":
        include_throughput = False

    for index in module.params.get("indexes"):
        if index.get("type") not in ["global_all", "global_include", "global_keys_only"]:
            continue
        name = index.get("name")
        if name in index_exists:
            module.fail_json(msg=f"Duplicate key {name} in list of global indexes")
        # Convert the type name to upper case and remove the global_
        index["type"] = index["type"].upper()[7:]
        index = _generate_index(index, include_throughput)
        index_exists[name] = True
        indexes.append(index)

    return indexes


def _generate_local_indexes():
    index_exists = dict()
    indexes = list()

    for index in module.params.get("indexes"):
        if index.get("type") not in ["all", "include", "keys_only"]:
            continue
        name = index.get("name")
        if name in index_exists:
            module.fail_json(msg=f"Duplicate key {name} in list of local indexes")
        index["type"] = index["type"].upper()
        index = _generate_index(index, False)
        index_exists[name] = True
        indexes.append(index)

    return indexes


def _generate_global_index_map(current_table):
    global_index_map = dict()
    existing_indexes = current_table["_global_index_map"]
    for index in module.params.get("indexes"):
        if index.get("type") not in ["global_all", "global_include", "global_keys_only"]:
            continue
        name = index.get("name")
        if name in global_index_map:
            module.fail_json(msg=f"Duplicate key {name} in list of global indexes")
        idx = _merge_index_params(index, existing_indexes.get(name, {}))
        # Convert the type name to upper case and remove the global_
        idx["type"] = idx["type"].upper()[7:]
        global_index_map[name] = idx
    return global_index_map


def _generate_local_index_map(current_table):
    local_index_map = dict()
    existing_indexes = current_table["_local_index_map"]
    for index in module.params.get("indexes"):
        if index.get("type") not in ["all", "include", "keys_only"]:
            continue
        name = index.get("name")
        if name in local_index_map:
            module.fail_json(msg=f"Duplicate key {name} in list of local indexes")
        idx = _merge_index_params(index, existing_indexes.get(name, {}))
        # Convert the type name to upper case
        idx["type"] = idx["type"].upper()
        local_index_map[name] = idx
    return local_index_map


def _generate_index(index, include_throughput=True):
    key_schema = _generate_schema(index)
    throughput = _generate_throughput(index)
    non_key_attributes = index["includes"] or []
    projection = dict(
        ProjectionType=index["type"],
    )
    if index["type"] != "ALL":
        if non_key_attributes:
            projection["NonKeyAttributes"] = non_key_attributes
    else:
        if non_key_attributes:
            module.fail_json(
                "DynamoDB does not support specifying non-key-attributes ('includes') for indexes of type 'all'. Index"
                f" name: {index['name']}"
            )

    idx = dict(
        IndexName=index["name"],
        KeySchema=key_schema,
        Projection=projection,
    )

    if include_throughput:
        idx["ProvisionedThroughput"] = throughput

    return idx


def _attribute_changes(current_table):
    # TODO (future) It would be nice to catch attempts to change types here.
    return _generate_attributes()


def _global_index_changes(current_table):
    current_global_index_map = current_table["_global_index_map"]
    global_index_map = _generate_global_index_map(current_table)

    current_billing_mode = current_table.get("billing_mode")

    if module.params.get("billing_mode") is None:
        billing_mode = current_billing_mode
    else:
        billing_mode = module.params.get("billing_mode")

    include_throughput = True

    if billing_mode == "PAY_PER_REQUEST":
        include_throughput = False

    index_changes = list()

    # TODO (future) it would be nice to add support for deleting an index
    for name, current_value in global_index_map.items():
        idx = dict(_generate_index(current_value, include_throughput=include_throughput))
        if name not in current_global_index_map:
            index_changes.append(dict(Create=idx))
        else:
            # The only thing we can change is the provisioned throughput.
            # TODO (future) it would be nice to throw a deprecation here
            # rather than dropping other changes on the floor
            _current = current_global_index_map[name]
            _new = current_value

            if include_throughput:
                change = dict(_throughput_changes(_current, _new))
                if change:
                    update = dict(
                        IndexName=name,
                        ProvisionedThroughput=change,
                    )
                    index_changes.append(dict(Update=update))

    return index_changes


def _local_index_changes(current_table):
    # TODO (future) Changes to Local Indexes aren't possible after creation,
    # we should probably throw a deprecation warning here (original module
    # also just dropped these changes on the floor)
    return []


def _update_table(current_table):
    changes = dict()
    additional_global_index_changes = list()

    # Get throughput / billing_mode changes
    throughput_changes = _throughput_changes(current_table)
    if throughput_changes:
        changes["ProvisionedThroughput"] = throughput_changes

    current_billing_mode = current_table.get("billing_mode")
    new_billing_mode = module.params.get("billing_mode")

    if new_billing_mode is None:
        new_billing_mode = current_billing_mode

    if current_billing_mode != new_billing_mode:
        changes["BillingMode"] = new_billing_mode

    # Update table_class use exisiting if none is defined
    if module.params.get("table_class"):
        if module.params.get("table_class") != current_table.get("table_class"):
            changes["TableClass"] = module.params.get("table_class")

    global_index_changes = _global_index_changes(current_table)
    if global_index_changes:
        changes["GlobalSecondaryIndexUpdates"] = global_index_changes
        # Only one index can be changed at a time except if changing the billing mode, pass the first during the
        # main update and deal with the others on a slow retry to wait for
        # completion

        if current_billing_mode == new_billing_mode:
            if len(global_index_changes) > 1:
                changes["GlobalSecondaryIndexUpdates"] = [global_index_changes[0]]
                additional_global_index_changes = global_index_changes[1:]

    local_index_changes = _local_index_changes(current_table)
    if local_index_changes:
        changes["LocalSecondaryIndexUpdates"] = local_index_changes

    if not changes:
        return False

    if module.check_mode:
        return True

    if global_index_changes or local_index_changes:
        changes["AttributeDefinitions"] = _generate_attributes()

    try:
        client.update_table(aws_retry=True, TableName=module.params.get("name"), **changes)
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
        module.fail_json_aws(e, msg="Failed to update table")

    if additional_global_index_changes:
        for index in additional_global_index_changes:
            wait_indexes()
            try:
                _update_table_with_long_retry(
                    GlobalSecondaryIndexUpdates=[index], AttributeDefinitions=changes["AttributeDefinitions"]
                )
            except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
                module.fail_json_aws(
                    e,
                    msg="Failed to update table",
                    changes=changes,
                    additional_global_index_changes=additional_global_index_changes,
                )

    return True


def _update_tags(current_table):
    _tags = module.params.get("tags")
    if _tags is None:
        return False

    tags_to_add, tags_to_remove = compare_aws_tags(
        current_table["tags"], module.params.get("tags"), purge_tags=module.params.get("purge_tags")
    )

    # If neither need updating we can return already
    if not (tags_to_add or tags_to_remove):
        return False

    if module.check_mode:
        return True

    if tags_to_add:
        try:
            client.tag_resource(
                aws_retry=True,
                ResourceArn=current_table["arn"],
                Tags=ansible_dict_to_boto3_tag_list(tags_to_add),
            )
        except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
            module.fail_json_aws(e, msg="Failed to tag table")
    if tags_to_remove:
        try:
            client.untag_resource(
                aws_retry=True,
                ResourceArn=current_table["arn"],
                TagKeys=tags_to_remove,
            )
        except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
            module.fail_json_aws(e, msg="Failed to untag table")

    return True


def update_table(current_table):
    primary_index_changes = _primary_index_changes(current_table)
    if primary_index_changes:
        module.fail_json(
            f"DynamoDB does not support updating the Primary keys on a table. Changed paramters are: {primary_index_changes}"
        )

    changed = False
    changed |= _update_table(current_table)
    changed |= _update_tags(current_table)

    if module.params.get("wait"):
        wait_exists()
        wait_indexes()

    return changed


def create_table():
    table_name = module.params.get("name")
    table_class = module.params.get("table_class")
    hash_key_name = module.params.get("hash_key_name")
    billing_mode = module.params.get("billing_mode")

    if billing_mode is None:
        billing_mode = "PROVISIONED"

    tags = ansible_dict_to_boto3_tag_list(module.params.get("tags") or {})

    if not hash_key_name:
        module.fail_json('"hash_key_name" must be provided when creating a new table.')

    if module.check_mode:
        return True

    if billing_mode == "PROVISIONED":
        throughput = _generate_throughput()

    attributes = _generate_attributes()
    key_schema = _generate_schema()
    local_indexes = _generate_local_indexes()
    global_indexes = _generate_global_indexes(billing_mode)

    params = dict(
        TableName=table_name,
        AttributeDefinitions=attributes,
        KeySchema=key_schema,
        Tags=tags,
        BillingMode=billing_mode,
        # TODO (future)
        # StreamSpecification,
        # SSESpecification,
    )

    if table_class:
        params["TableClass"] = table_class
    if billing_mode == "PROVISIONED":
        params["ProvisionedThroughput"] = throughput
    if local_indexes:
        params["LocalSecondaryIndexes"] = local_indexes
    if global_indexes:
        params["GlobalSecondaryIndexes"] = global_indexes

    try:
        client.create_table(aws_retry=True, **params)
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
        module.fail_json_aws(e, msg="Failed to create table")

    if module.params.get("wait"):
        wait_exists()
        wait_indexes()

    return True


def delete_table(current_table):
    if not current_table:
        return False

    if module.check_mode:
        return True

    table_name = module.params.get("name")

    # If an index is mid-update then we have to wait for the update to complete
    # before deletion will succeed
    long_retry = AWSRetry.jittered_backoff(
        retries=45,
        delay=5,
        max_delay=30,
        catch_extra_error_codes=["LimitExceededException", "ResourceInUseException"],
    )

    try:
        long_retry(client.delete_table)(TableName=table_name)
    except is_boto3_error_code("ResourceNotFoundException"):
        return False
    except (
        botocore.exceptions.ClientError,
        botocore.exceptions.BotoCoreError,
    ) as e:  # pylint: disable=duplicate-except
        module.fail_json_aws(e, msg="Failed to delete table")

    if module.params.get("wait"):
        wait_not_exists()

    return True


def main():
    global module
    global client

    # TODO (future) It would be good to split global and local indexes.  They have
    # different parameters, use a separate namespace for names,
    #  and local indexes can't be updated.
    index_options = dict(
        name=dict(type="str", required=True),
        # It would be nice to make this optional, but because Local and Global
        # indexes are mixed in here we need this to be able to tell to which
        # group of indexes the index belongs.
        type=dict(type="str", required=True, choices=INDEX_TYPE_OPTIONS),
        hash_key_name=dict(type="str", required=False),
        hash_key_type=dict(type="str", required=False, choices=KEY_TYPE_CHOICES),
        range_key_name=dict(type="str", required=False),
        range_key_type=dict(type="str", required=False, choices=KEY_TYPE_CHOICES),
        includes=dict(type="list", required=False, elements="str"),
        read_capacity=dict(type="int", required=False),
        write_capacity=dict(type="int", required=False),
    )

    argument_spec = dict(
        state=dict(default="present", choices=["present", "absent"]),
        name=dict(required=True, type="str"),
        hash_key_name=dict(type="str"),
        hash_key_type=dict(type="str", choices=KEY_TYPE_CHOICES),
        range_key_name=dict(type="str"),
        range_key_type=dict(type="str", choices=KEY_TYPE_CHOICES),
        billing_mode=dict(type="str", choices=["PROVISIONED", "PAY_PER_REQUEST"]),
        read_capacity=dict(type="int"),
        write_capacity=dict(type="int"),
        indexes=dict(default=[], type="list", elements="dict", options=index_options),
        table_class=dict(type="str", choices=["STANDARD", "STANDARD_INFREQUENT_ACCESS"]),
        tags=dict(type="dict", aliases=["resource_tags"]),
        purge_tags=dict(type="bool", default=True),
        wait=dict(type="bool", default=True),
        wait_timeout=dict(default=900, type="int", aliases=["wait_for_active_timeout"]),
    )

    module = AnsibleAWSModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        check_boto3=False,
    )

    retry_decorator = AWSRetry.jittered_backoff(
        catch_extra_error_codes=["LimitExceededException", "ResourceInUseException", "ResourceNotFoundException"],
    )
    client = module.client("dynamodb", retry_decorator=retry_decorator)

    current_table = get_dynamodb_table()
    changed = False
    table = None
    results = dict()

    state = module.params.get("state")
    if state == "present":
        if current_table:
            changed |= update_table(current_table)
        else:
            changed |= create_table()
        table = get_dynamodb_table()
    elif state == "absent":
        changed |= delete_table(current_table)

    compat_results = compatability_results(table)
    if compat_results:
        results.update(compat_results)

    results["changed"] = changed
    if table:
        # These are used to pass computed data about, not needed for users
        table.pop("_global_index_map", None)
        table.pop("_local_index_map", None)
        results["table"] = table

    module.exit_json(**results)


if __name__ == "__main__":
    main()



# This code is part of the Ansible collection community.docker, but is an independent component.
# This particular file, and this file only, is based on the Docker SDK for Python (https://github.com/docker/docker-py/)
#
# Copyright (c) 2016-2022 Docker, Inc.
#
# It is licensed under the Apache 2.0 license (see LICENSES/Apache-2.0.txt in this collection)
# SPDX-License-Identifier: Apache-2.0

# Note that this module util is **PRIVATE** to the collection. It can have breaking changes at any time.
# Do not use this from other collections or standalone plugins/modules!

from __future__ import annotations

import os
from shutil import which


def find_executable(executable: str, path: str | None = None) -> str | None:
    """
    As distutils.spawn.find_executable, but on Windows, look up
    every extension declared in PATHEXT instead of just `.exe`
    """
    # shutil.which() already uses PATHEXT on Windows, so on
    # Python 3 we can simply use shutil.which() in all cases.
    # (https://github.com/docker/docker-py/commit/42789818bed5d86b487a030e2e60b02bf0cfa284)
    return which(executable, path=path)


def create_environment_dict(overrides: dict[str, str] | None) -> dict[str, str]:
    """
    Create and return a copy of os.environ with the specified overrides
    """
    result = os.environ.copy()
    result.update(overrides or {})
    return result



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Ansible module to manage CheckPoint Firewall (c) 2019
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {
    "metadata_version": "1.1",
    "status": ["preview"],
    "supported_by": "community",
}

DOCUMENTATION = """
---
module: cp_mgmt_reset_sic
short_description: Reset Secure Internal Communication (SIC). To complete the reset operation need also to reset the device in the Check Point Configuration
                   Tool (by running cpconfig in Clish or Expert mode). Communication will not be possible until you reset and re-initialize the device properly.
description:
  - Reset Secure Internal Communication (SIC). To complete the reset operation need also to reset the device in the Check Point Configuration Tool (by
    running cpconfig in Clish or Expert mode). Communication will not be possible until you reset and re-initialize the device properly.
  - All operations are performed over Web Services API.
  - Available from R81.20 management version.
version_added: "3.0.0"
author: "Eden Brillant (@chkp-edenbr)"
options:
  name:
    description:
      - Gateway, cluster member or Check Point host name.
    type: str
  auto_publish_session:
    description:
    - Publish the current session if changes have been performed after task completes.
    type: bool
extends_documentation_fragment: check_point.mgmt.checkpoint_commands
"""

EXAMPLES = """
- name: reset-sic
  cp_mgmt_reset_sic:
    name: gw1
"""

RETURN = """
cp_mgmt_reset_sic:
  description: The checkpoint reset-sic output.
  returned: always.
  type: dict
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.check_point.mgmt.plugins.module_utils.checkpoint import (
    checkpoint_argument_spec_for_commands,
    api_command,
)


def main():
    argument_spec = dict(
        name=dict(type="str"), auto_publish_session=dict(type="bool")
    )
    argument_spec.update(checkpoint_argument_spec_for_commands)

    module = AnsibleModule(argument_spec=argument_spec)

    command = "reset-sic"

    result = api_command(module, command)
    module.exit_json(**result)


if __name__ == "__main__":
    main()



#!/usr/bin/python

# Copyright Ansible Project
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: github_key
short_description: Manage GitHub access keys
description:
  - Creates, removes, or updates GitHub access keys.
  - Works with both GitHub.com and GitHub Enterprise Server installations.
extends_documentation_fragment:
  - community.general._attributes
attributes:
  check_mode:
    support: full
  diff_mode:
    support: none
options:
  token:
    description:
      - GitHub Access Token with permission to list and create public keys.
    required: true
    type: str
  name:
    description:
      - SSH key name.
    required: true
    type: str
  pubkey:
    description:
      - SSH public key value. Required when O(state=present).
    type: str
  state:
    description:
      - Whether to remove a key, ensure that it exists, or update its value.
    choices: ['present', 'absent']
    default: 'present'
    type: str
  force:
    description:
      - The default is V(true), which replaces the existing remote key if it is different than O(pubkey). If V(false), the
        key is only set if no key with the given O(name) exists.
    type: bool
    default: true
  api_url:
    description:
      - URL to the GitHub API if not using github.com but your own GitHub Enterprise instance.
    type: str
    default: 'https://api.github.com'
    version_added: "11.0.0"

author: Robert Estelle (@erydo)
"""

RETURN = r"""
deleted_keys:
  description: An array of key objects that were deleted. Only present on state=absent.
  type: list
  returned: When state=absent
  sample:
    [
      {
        "id": 0,
        "key": "BASE64 encoded key",
        "url": "http://example.com/github key",
        "created_at": "YYYY-MM-DDTHH:MM:SZ",
        "read_only": false
      }
    ]
matching_keys:
  description: An array of keys matching the specified name. Only present on state=present.
  type: list
  returned: When state=present
  sample:
    [
      {
        "id": 0,
        "key": "BASE64 encoded key",
        "url": "http://example.com/github key",
        "created_at": "YYYY-MM-DDTHH:MM:SZ",
        "read_only": false
      }
    ]
key:
  description: Metadata about the key just created. Only present on state=present.
  type: dict
  returned: success
  sample:
    {
      "id": 0,
      "key": "BASE64 encoded key",
      "url": "http://example.com/github key",
      "created_at": "YYYY-MM-DDTHH:MM:SZ",
      "read_only": false
    }
"""

EXAMPLES = r"""
- name: Read SSH public key to authorize
  ansible.builtin.shell: cat /home/foo/.ssh/id_rsa.pub
  register: ssh_pub_key

- name: Authorize key with GitHub
  local_action:
    module: github_key
    name: Access Key for Some Machine
    token: '{{ github_access_token }}'
    pubkey: '{{ ssh_pub_key.stdout }}'

# Alternatively, a single task can be used reading a key from a file on the controller
- name: Authorize key with GitHub
  community.general.github_key:
    name: Access Key for Some Machine
    token: '{{ github_access_token }}'
    pubkey: "{{ lookup('ansible.builtin.file', '/home/foo/.ssh/id_rsa.pub') }}"

# GitHub Enterprise Server usage
- name: Authorize key with GitHub Enterprise
  community.general.github_key:
    name: Access Key for Some Machine
    token: '{{ github_enterprise_token }}'
    pubkey: "{{ lookup('ansible.builtin.file', '/home/foo/.ssh/id_rsa.pub') }}"
    api_url: 'https://github.company.com/api/v3'
"""

import datetime
import json
import re

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.urls import fetch_url

from ansible_collections.community.general.plugins.module_utils._datetime import (
    now,
)


class GitHubResponse:
    def __init__(self, response, info):
        self.content = response.read()
        self.info = info

    def json(self):
        return json.loads(self.content)

    def links(self):
        links = {}
        if "link" in self.info:
            link_header = self.info["link"]
            matches = re.findall('<([^>]+)>; rel="([^"]+)"', link_header)
            for url, rel in matches:
                links[rel] = url
        return links


class GitHubSession:
    def __init__(self, module, token, api_url):
        self.module = module
        self.token = token
        self.api_url = api_url.rstrip("/")

    def request(self, method, url, data=None):
        headers = {
            "Authorization": f"token {self.token}",
            "Content-Type": "application/json",
            "Accept": "application/vnd.github.v3+json",
        }
        response, info = fetch_url(self.module, url, method=method, data=data, headers=headers)
        if not (200 <= info["status"] < 400):
            self.module.fail_json(msg=f" failed to send request {method} to {url}: {info['msg']}")
        return GitHubResponse(response, info)


def get_all_keys(session):
    url = f"{session.api_url}/user/keys"
    result = []
    while url:
        r = session.request("GET", url)
        result.extend(r.json())
        url = r.links().get("next")
    return result


def create_key(session, name, pubkey, check_mode):
    if check_mode:
        now_t = now()
        return {
            "id": 0,
            "key": pubkey,
            "title": name,
            "url": "http://example.com/CHECK_MODE_GITHUB_KEY",
            "created_at": datetime.datetime.strftime(now_t, "%Y-%m-%dT%H:%M:%SZ"),
            "read_only": False,
            "verified": False,
        }
    else:
        return session.request(
            "POST", f"{session.api_url}/user/keys", data=json.dumps({"title": name, "key": pubkey})
        ).json()


def delete_keys(session, to_delete, check_mode):
    if check_mode:
        return

    for key in to_delete:
        session.request("DELETE", f"{session.api_url}/user/keys/{key['id']}")


def ensure_key_absent(session, name, check_mode):
    to_delete = [key for key in get_all_keys(session) if key["title"] == name]
    delete_keys(session, to_delete, check_mode=check_mode)

    return {"changed": bool(to_delete), "deleted_keys": to_delete}


def ensure_key_present(module, session, name, pubkey, force, check_mode):
    all_keys = get_all_keys(session)
    matching_keys = [k for k in all_keys if k["title"] == name]
    deleted_keys = []

    new_signature = pubkey.split(" ")[1]
    for key in all_keys:
        existing_signature = key["key"].split(" ")[1]
        if new_signature == existing_signature and key["title"] != name:
            module.fail_json(
                msg=f"another key with the same content is already registered under the name |{key['title']}|"
            )

    if matching_keys and force and matching_keys[0]["key"].split(" ")[1] != new_signature:
        delete_keys(session, matching_keys, check_mode=check_mode)
        (deleted_keys, matching_keys) = (matching_keys, [])

    if not matching_keys:
        key = create_key(session, name, pubkey, check_mode=check_mode)
    else:
        key = matching_keys[0]

    return {
        "changed": bool(deleted_keys or not matching_keys),
        "deleted_keys": deleted_keys,
        "matching_keys": matching_keys,
        "key": key,
    }


def main():
    argument_spec = {
        "token": {"required": True, "no_log": True},
        "name": {"required": True},
        "pubkey": {},
        "state": {"choices": ["present", "absent"], "default": "present"},
        "force": {"default": True, "type": "bool"},
        "api_url": {"default": "https://api.github.com", "type": "str"},
    }
    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
    )

    token = module.params["token"]
    name = module.params["name"]
    state = module.params["state"]
    force = module.params["force"]
    pubkey = module.params.get("pubkey")
    api_url = module.params.get("api_url")

    if pubkey:
        pubkey_parts = pubkey.split(" ")
        # Keys consist of a protocol, the key data, and an optional comment.
        if len(pubkey_parts) < 2:
            module.fail_json(msg='"pubkey" parameter has an invalid format')
    elif state == "present":
        module.fail_json(msg='"pubkey" is required when state=present')

    session = GitHubSession(module, token, api_url)
    if state == "present":
        result = ensure_key_present(module, session, name, pubkey, force=force, check_mode=module.check_mode)
    elif state == "absent":
        result = ensure_key_absent(session, name, check_mode=module.check_mode)

    module.exit_json(**result)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2020, Kevin Breit (@kbreit) <kevin.breit@kevinbreit.net>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

ANSIBLE_METADATA = {
    'metadata_version': '1.1',
    "status": ['deprecated'],
    'supported_by': 'community'
}

DOCUMENTATION = r'''
---
module: meraki_ms_stack_l3_interface
short_description: Manage routed interfaces on MS switches
description:
- Allows for creation, management, and visibility into routed interfaces on Meraki MS switches.
notes:
- Once a layer 3 interface is created, the API does not allow updating the interface and specifying C(default_gateway).
deprecated:
  removed_in: '3.0.0'
  why: Updated modules released with increased functionality
  alternative: cisco.meraki.networks_switch_stacks_routing_interfaces
options:
    state:
        description:
        - Create or modify an organization.
        type: str
        choices: [ present, query, absent ]
        default: present
    net_name:
      description:
      - Name of network which configuration is applied to.
      aliases: [network]
      type: str
    net_id:
      description:
      - ID of network which configuration is applied to.
      type: str
    stack_id:
        description:
        - The unique identifier of the stack.
        type: str
    vlan_id:
        description:
        - The VLAN this routed interface is on.
        - VLAN must be between 1 and 4094.
        type: int
    default_gateway:
        description:
        - The next hop for any traffic that isn't going to a directly connected subnet or over a static route.
        - This IP address must exist in a subnet with a routed interface.
        type: str
    interface_ip:
        description:
        - The IP address this switch will use for layer 3 routing on this VLAN or subnet.
        - This cannot be the same as the switch's management IP.
        type: str
    interface_id:
        description:
        - Uniqiue identification number for layer 3 interface.
        type: str
    multicast_routing:
        description:
        - Enable multicast support if multicast routing between VLANs is required.
        type: str
        choices: [disabled, enabled, IGMP snooping querier]
    name:
        description:
        - A friendly name or description for the interface or VLAN.
        type: str
    subnet:
        description:
        - The network that this routed interface is on, in CIDR notation.
        type: str
    ospf_settings:
        description:
        - The OSPF routing settings of the interface.
        type: dict
        suboptions:
            cost:
                description:
                - The path cost for this interface.
                type: int
            area:
                description:
                - The OSPF area to which this interface should belong.
                - Can be either 'disabled' or the identifier of an existing OSPF area.
                type: str
            is_passive_enabled:
                description:
                - When enabled, OSPF will not run on the interface, but the subnet will still be advertised.
                type: bool
author:
- Kevin Breit (@kbreit)
extends_documentation_fragment: cisco.meraki.meraki
'''

EXAMPLES = r'''
- name: Query all l3 interfaces
  meraki_ms_stack_l3_interface:
    auth_key: abc123
    state: query
    serial: aaa-bbb-ccc

- name: Query one l3 interface
  meraki_ms_stack_l3_interface:
    auth_key: abc123
    state: query
    serial: aaa-bbb-ccc
    name: Test L3 interface

- name: Create l3 interface
  meraki_ms_stack_l3_interface:
    auth_key: abc123
    state: present
    serial: aaa-bbb-ccc
    name: "Test L3 interface 2"
    subnet: "192.168.3.0/24"
    interface_ip: "192.168.3.2"
    multicast_routing: disabled
    vlan_id: 11
    ospf_settings:
      area: 0
      cost: 1
      is_passive_enabled: true

- name: Update l3 interface
  meraki_ms_stack_l3_interface:
    auth_key: abc123
    state: present
    serial: aaa-bbb-ccc
    name: "Test L3 interface 2"
    subnet: "192.168.3.0/24"
    interface_ip: "192.168.3.2"
    multicast_routing: disabled
    vlan_id: 11
    ospf_settings:
      area: 0
      cost: 2
      is_passive_enabled: true

- name: Delete l3 interface
  meraki_ms_stack_l3_interface:
    auth_key: abc123
    state: absent
    serial: aaa-bbb-ccc
    interface_id: abc123344566
'''

RETURN = r'''
data:
    description: Information about the layer 3 interfaces.
    returned: success
    type: complex
    contains:
        vlan_id:
            description: The VLAN this routed interface is on.
            returned: success
            type: int
            sample: 10
        default_gateway:
            description: The next hop for any traffic that isn't going to a directly connected subnet or over a static route.
            returned: success
            type: str
            sample: 192.168.2.1
        interface_ip:
            description: The IP address this switch will use for layer 3 routing on this VLAN or subnet.
            returned: success
            type: str
            sample: 192.168.2.2
        interface_id:
            description: Uniqiue identification number for layer 3 interface.
            returned: success
            type: str
            sample: 62487444811111120
        multicast_routing:
            description: Enable multicast support if multicast routing between VLANs is required.
            returned: success
            type: str
            sample: disabled
        name:
            description: A friendly name or description for the interface or VLAN.
            returned: success
            type: str
            sample: L3 interface
        subnet:
            description: The network that this routed interface is on, in CIDR notation.
            returned: success
            type: str
            sample: 192.168.2.0/24
        ospf_settings:
            description: The OSPF routing settings of the interface.
            returned: success
            type: complex
            contains:
                cost:
                    description: The path cost for this interface.
                    returned: success
                    type: int
                    sample: 1
                area:
                    description: The OSPF area to which this interface should belong.
                    returned: success
                    type: str
                    sample: 0
                is_passive_enabled:
                    description: When enabled, OSPF will not run on the interface, but the subnet will still be advertised.
                    returned: success
                    type: bool
                    sample: true
'''

from ansible.module_utils.basic import AnsibleModule, json
from ansible_collections.cisco.meraki.plugins.module_utils.network.meraki.meraki import MerakiModule, meraki_argument_spec


def construct_payload(meraki):
    payload = {}
    if meraki.params['name'] is not None:
        payload['name'] = meraki.params['name']
    if meraki.params['subnet'] is not None:
        payload['subnet'] = meraki.params['subnet']
    if meraki.params['interface_ip'] is not None:
        payload['interfaceIp'] = meraki.params['interface_ip']
    if meraki.params['multicast_routing'] is not None:
        payload['multicastRouting'] = meraki.params['multicast_routing']
    if meraki.params['vlan_id'] is not None:
        payload['vlanId'] = meraki.params['vlan_id']
    if meraki.params['default_gateway'] is not None:
        payload['defaultGateway'] = meraki.params['default_gateway']
    if meraki.params['ospf_settings'] is not None:
        payload['ospfSettings'] = {}
        if meraki.params['ospf_settings']['area'] is not None:
            payload['ospfSettings']['area'] = meraki.params['ospf_settings']['area']
        if meraki.params['ospf_settings']['cost'] is not None:
            payload['ospfSettings']['cost'] = meraki.params['ospf_settings']['cost']
        if meraki.params['ospf_settings']['is_passive_enabled'] is not None:
            payload['ospfSettings']['isPassiveEnabled'] = meraki.params['ospf_settings']['is_passive_enabled']
    return payload


def get_interface_id(meraki, data, name):
    # meraki.fail_json(msg=data)
    for interface in data:
        if interface['name'] == name:
            return interface['interfaceId']
    return None


def get_interface(interfaces, interface_id):
    for interface in interfaces:
        if interface['interfaceId'] == interface_id:
            return interface
    return None


def main():
    # define the available arguments/parameters that a user can pass to
    # the module

    ospf_arg_spec = dict(area=dict(type='str'),
                         cost=dict(type='int'),
                         is_passive_enabled=dict(type='bool'),
                         )

    argument_spec = meraki_argument_spec()
    argument_spec.update(state=dict(type='str', choices=['present', 'query', 'absent'], default='present'),
                         net_id=dict(type='str'),
                         net_name=dict(type='str', aliases=['network']),
                         stack_id=dict(type='str'),
                         name=dict(type='str'),
                         subnet=dict(type='str'),
                         interface_id=dict(type='str'),
                         interface_ip=dict(type='str'),
                         multicast_routing=dict(type='str', choices=['disabled', 'enabled', 'IGMP snooping querier']),
                         vlan_id=dict(type='int'),
                         default_gateway=dict(type='str'),
                         ospf_settings=dict(type='dict', default=None, options=ospf_arg_spec),
                         )

    # the AnsibleModule object will be our abstraction working with Ansible
    # this includes instantiation, a couple of common attr would be the
    # args/params passed to the execution, as well as if the module
    # supports check mode
    module = AnsibleModule(argument_spec=argument_spec,
                           supports_check_mode=True,
                           )
    meraki = MerakiModule(module, function='ms_stack_l3_interfaces')

    meraki.params['follow_redirects'] = 'all'

    query_urls = {'ms_stack_l3_interfaces': '/networks/{net_id}/switch/stacks/{stack_id}/routing/interfaces'}
    query_one_urls = {'ms_stack_l3_interfaces': '/networks/{net_id}/switch/stacks/{stack_id}/routing/interfaces'}
    create_urls = {'ms_stack_l3_interfaces': '/networks/{net_id}/switch/stacks/{stack_id}/routing/interfaces'}
    update_urls = {'ms_stack_l3_interfaces': '/networks/{net_id}/switch/stacks/{stack_id}/routing/interfaces/{interface_id}'}
    delete_urls = {'ms_stack_l3_interfaces': '/networks/{net_id}/switch/stacks/{stack_id}/routing/interfaces/{interface_id}'}

    meraki.url_catalog['get_all'].update(query_urls)
    meraki.url_catalog['get_one'].update(query_one_urls)
    meraki.url_catalog['create'] = create_urls
    meraki.url_catalog['update'] = update_urls
    meraki.url_catalog['delete'] = delete_urls

    payload = None

    if meraki.params['vlan_id'] is not None:
        if meraki.params['vlan_id'] < 1 or meraki.params['vlan_id'] > 4094:
            meraki.fail_json(msg='vlan_id must be between 1 and 4094')

    org_id = meraki.params['org_id']
    if org_id is None:
        orgs = meraki.get_orgs()
        for org in orgs:
            if org['name'] == meraki.params['org_name']:
                org_id = org['id']
    net_id = meraki.params['net_id']
    if net_id is None:
        net_id = meraki.get_net_id(net_name=meraki.params['net_name'],
                                   data=meraki.get_nets(org_id=org_id))

    interface_id = meraki.params['interface_id']
    interfaces = None
    if interface_id is None:
        if meraki.params['name'] is not None:
            path = meraki.construct_path('get_all', net_id=net_id, custom={'stack_id': meraki.params['stack_id']})
            interfaces = meraki.request(path, method='GET')
            interface_id = get_interface_id(meraki, interfaces, meraki.params['name'])

    # manipulate or modify the state as needed (this is going to be the
    # part where your module will do what it needs to do)

    if meraki.params['state'] == 'query':
        if interface_id is not None:  # Query one interface
            path = meraki.construct_path('get_one', net_id=net_id, custom={'stack_id': meraki.params['stack_id'],
                                                                           'interface_id': interface_id})
            response = meraki.request(path, method='GET')
            meraki.result['data'] = response
            meraki.exit_json(**meraki.result)
        else:  # Query all interfaces
            path = meraki.construct_path('get_all', net_id=net_id, custom={'stack_id': meraki.params['stack_id']})
            response = meraki.request(path, method='GET')
            meraki.result['data'] = response
            meraki.exit_json(**meraki.result)
    elif meraki.params['state'] == 'present':
        if interface_id is None:  # Create a new interface
            payload = construct_payload(meraki)
            if meraki.check_mode is True:
                meraki.result['data'] = payload
                meraki.result['changed'] = True
                meraki.exit_json(**meraki.result)
            path = meraki.construct_path('create', net_id=net_id, custom={'stack_id': meraki.params['stack_id']})
            response = meraki.request(path, method='POST', payload=json.dumps(payload))
            meraki.result['data'] = response
            meraki.result['changed'] = True
            meraki.exit_json(**meraki.result)
        else:
            if interfaces is None:
                path = meraki.construct_path('get_all', net_id=net_id, custom={'stack_id': meraki.params['stack_id']})
                interfaces = meraki.request(path, method='GET')
            payload = construct_payload(meraki)
            interface = get_interface(interfaces, interface_id)
            if meraki.is_update_required(interface, payload):
                if meraki.check_mode is True:
                    interface.update(payload)
                    meraki.result['data'] = interface
                    meraki.result['changed'] = True
                    meraki.exit_json(**meraki.result)
                path = meraki.construct_path('update', net_id=net_id, custom={'stack_id': meraki.params['stack_id'],
                                                                              'interface_id': interface_id})
                response = meraki.request(path, method='PUT', payload=json.dumps(payload))
                meraki.result['data'] = response
                meraki.result['changed'] = True
                meraki.exit_json(**meraki.result)
            else:
                meraki.result['data'] = interface
                meraki.exit_json(**meraki.result)
    elif meraki.params['state'] == 'absent':
        if meraki.check_mode is True:
            meraki.result['data'] = {}
            meraki.result['changed'] = True
            meraki.exit_json(**meraki.result)
        path = meraki.construct_path('delete', net_id=net_id, custom={'stack_id': meraki.params['stack_id'],
                                                                      'interface_id': meraki.params['interface_id']})
        response = meraki.request(path, method='DELETE')
        meraki.result['data'] = response
        meraki.result['changed'] = True

    # in the event of a successful module execution, you will want to
    # simple AnsibleModule.exit_json(), passing the key/value results
    meraki.exit_json(**meraki.result)


if __name__ == '__main__':
    main()



#!/usr/bin/python
#
# Copyright (c) 2025 Klaas Demter (@Klaas-)
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

DOCUMENTATION = '''
---
module: azure_rm_monitordatacollectionrules_info
version_added: "3.7.0"
short_description: Get Data Collection Rules
description:
    - Get Data Collection Rules

options:
    name:
        description:
            - The name of the data collection rule you're trying to get details about.
        type: str
    resource_group:
        description:
            - The name of the resource group in which the data collection rule is (if you use name)
            - The name of the resource group where you want to list data collection rules (if you don't use name)
        type: str
extends_documentation_fragment:
    - azure.azcollection.azure

author:
    - Klaas Demter (@Klaas-)
'''

EXAMPLES = '''
- name: Get data collection rule details
  azure.azcollection.azure_rm_monitordatacollectionrules_info:
    name: DCRName
    resource_group: Resource_Group_Name

- name: Get all data collection rules in specific resource group
  azure.azcollection.azure_rm_monitordatacollectionrules_info:
    resource_group: Resource_Group_Name

- name: Get all data collection rules in the current subscription
  azure.azcollection.azure_rm_monitordatacollectionrules_info:
'''

RETURN = '''
datacollectionrules:
    description:
        - List of data collection rules
        - Can be empty if listing data collection rules
    type: list
    returned: always
    sample: [
        {
            "data_flows": [...],
            "data_sources": {},
            "description": "Description of your data collection rule",
            "destinations": {},
            "etag": "str",
            "id": \
"/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/resource_group_name/providers/Microsoft.Insights/dataCollectionRules/data_collection_rule_name",
            "immutable_id": "dcr-xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx",
            "kind": "Linux",
            "location": "westeurope",
            "name": "data_collection_rule_name",
            "provisioning_state": "Succeeded",
            "system_data": {
                "created_at": "2025-01-01T00:00:00.000000Z",
                "created_by": "xxx@domain.tld",
                "created_by_type": "User",
                "last_modified_at": "2025-01-01T00:00:00.000000Z",
                "last_modified_by": "xxx@domain.tld",
                "last_modified_by_type": "User"
            },
            "tags": {},
            "type": "Microsoft.Insights/dataCollectionRules"
        }
    ]
'''

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase

try:
    from azure.core.exceptions import ResourceNotFoundError, HttpResponseError

except ImportError:
    # This is handled in azure_rm_common
    pass

AZURE_OBJECT_CLASS = 'DataCollectionRules'


class AzureRMDataCollectionRulesInfo(AzureRMModuleBase):
    """Information class for an Azure RM Data Collection Rules"""

    def __init__(self):
        self.module_arg_spec = dict(
            name=dict(type='str', required=False),
            resource_group=dict(type='str', required=False),
        )

        self.required_by = {
            'name': 'resource_group'
        }

        self.name = None
        self.resource_group = None
        self.log_path = None
        self.log_mode = None

        self.results = dict(
            changed=False,
            datacollectionrules=[]
        )

        super(AzureRMDataCollectionRulesInfo, self).__init__(derived_arg_spec=self.module_arg_spec,
                                                             supports_check_mode=True,
                                                             supports_tags=False,
                                                             facts_module=True,
                                                             required_by=self.required_by)

    def exec_module(self, **kwargs):
        """Main module execution method"""

        for key in self.module_arg_spec:
            setattr(self, key, kwargs[key])

        if self.name:
            result = self.get_data_collection_rule()
        else:
            result = self.list_data_collection_rules()

        self.results['datacollectionrules'] = result

        return self.results

    def get_data_collection_rule(self):
        '''
        Gets the properties of the specified data collection rule.

        :return: List of Data Collection Rules
        '''
        self.log("Checking if data collection rule {0} in resource group {1} is present".format(self.name,
                                                                                                self.resource_group))

        result = []
        data_collection_rule = None

        try:
            data_collection_rule = self.monitor_management_client_data_collection_rules.data_collection_rules.get(data_collection_rule_name=self.name,
                                                                                                                  resource_group_name=self.resource_group)
        except ResourceNotFoundError as ex:
            self.log("Could not find data collection rule {0} in resource group {1}".format(self.name, self.resource_group))
            return []
        except HttpResponseError as ex:
            if ex.error.code == 'InvalidSubscriptionId':
                self.log("Could not find subscription id")
                return []
            else:
                raise Exception(ex)
        if data_collection_rule:
            result = [self.serialize_obj(data_collection_rule, AZURE_OBJECT_CLASS)]

        return result

    def list_data_collection_rules(self):
        '''
        Gets the properties of the specified data collection rules in resource group or subscription.

        :return: List of Data Collection Rules
        '''

        result = []
        data_collection_rules = None

        if self.resource_group:
            self.log("Checking if the data collection rules in resource group {0} are present".format(self.resource_group))
            data_collection_rules_mgmt_client = self.monitor_management_client_data_collection_rules.data_collection_rules
            data_collection_rules = data_collection_rules_mgmt_client.list_by_resource_group(resource_group_name=self.resource_group)
        else:
            self.log("Checking if the data collection rules are present in subscription")
            data_collection_rules = self.monitor_management_client_data_collection_rules.data_collection_rules.list_by_subscription()
        if data_collection_rules:
            # it seems the exception is thrown when iterating through data_collection_rules, not when setting data_collection_rules
            try:
                for item in data_collection_rules:
                    result.append(self.serialize_obj(item, AZURE_OBJECT_CLASS))
            except ResourceNotFoundError as ex:
                self.log("Could not find resource group {0}".format(self.resource_group))
            except HttpResponseError as ex:
                if ex.error.code == 'InvalidSubscriptionId':
                    self.log("Could not find subscription id")
                else:
                    raise Exception(ex)

        return result


def main():
    """Main execution"""
    AzureRMDataCollectionRulesInfo()


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: ec2_vpc_dhcp_option
version_added: 1.0.0
short_description: Manages DHCP Options, and can ensure the DHCP options for the given VPC match what's
  requested
description:
  - This module removes, or creates DHCP option sets, and can associate them to a VPC.
  - Optionally, a new DHCP Options set can be created that converges a VPC's existing
    DHCP option set with values provided.
  - When dhcp_options_id is provided, the module will
    1. remove (with state='absent')
    2. ensure tags are applied (if state='present' and tags are provided
    3. attach it to a VPC (if state='present' and a vpc_id is provided.
  - If any of the optional values are missing, they will either be treated
    as a no-op (i.e., inherit what already exists for the VPC)
  - To remove existing options while inheriting, supply an empty value
    (e.g. set ntp_servers to [] if you want to remove them from the VPC's options)
author:
  - "Joel Thompson (@joelthompson)"
options:
  domain_name:
    description:
      - The domain name to set in the DHCP option sets.
    type: str
  dns_servers:
    description:
      - A list of IP addresses to set the DNS servers for the VPC to.
    type: list
    elements: str
  ntp_servers:
    description:
      - List of hosts to advertise as NTP servers for the VPC.
    type: list
    elements: str
  netbios_name_servers:
    description:
      - List of hosts to advertise as NetBIOS servers.
    type: list
    elements: str
  netbios_node_type:
    description:
      - NetBIOS node type to advertise in the DHCP options.
        The AWS recommendation is to use 2 (when using netbios name services)
        U(https://docs.aws.amazon.com/AmazonVPC/latest/UserGuide/VPC_DHCP_Options.html)
    type: int
  vpc_id:
    description:
      - VPC ID to associate with the requested DHCP option set.
      - If no VPC ID is provided, and no matching option set is found then a new
        DHCP option set is created.
    type: str
  delete_old:
    description:
      - Whether to delete the old VPC DHCP option set when associating a new one.
      - This is primarily useful for debugging/development purposes when you
        want to quickly roll back to the old option set. Note that this setting
        will be ignored, and the old DHCP option set will be preserved, if it
        is in use by any other VPC. (Otherwise, AWS will return an error.)
    type: bool
    default: true
  inherit_existing:
    description:
      - For any DHCP options not specified in these parameters, whether to
        inherit them from the options set already applied to O(vpc_id), or to
        reset them to be empty.
    type: bool
    default: false
  dhcp_options_id:
    description:
      - The resource_id of an existing DHCP options set.
        If this is specified, then it will override other settings, except tags
        (which will be updated to match)
    type: str
  state:
    description:
      - create/assign or remove the DHCP options.
        If state is set to absent, then a DHCP options set matched either
        by id, or tags and options will be removed if possible.
    default: present
    choices: [ 'absent', 'present' ]
    type: str
notes:
  - Support for O(purge_tags) was added in release 2.0.0.
extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.tags
  - amazon.aws.boto3
"""

RETURN = r"""
changed:
    description: Whether the dhcp options were changed
    type: bool
    returned: always
dhcp_options:
    description: The DHCP options created, associated or found
    returned: when available
    type: dict
    contains:
        dhcp_configurations:
            description: The DHCP configuration for the option set
            type: list
            sample:
              - '{"key": "ntp-servers", "values": [{"value": "10.0.0.2" , "value": "10.0.1.2"}]}'
              - '{"key": "netbios-name-servers", "values": [{value": "10.0.0.1"}, {"value": "10.0.1.1" }]}'
        dhcp_options_id:
            description: The aws resource id of the primary DCHP options set created or found
            type: str
            sample: "dopt-0955331de6a20dd07"
        owner_id:
            description: The ID of the AWS account that owns the DHCP options set.
            type: str
            sample: 012345678912
        tags:
            description: The tags to be applied to a DHCP options set
            type: list
            sample:
              - '{"Key": "CreatedBy", "Value": "ansible-test"}'
              - '{"Key": "Collection", "Value": "amazon.aws"}'
dhcp_options_id:
    description: The aws resource id of the primary DCHP options set created, found or removed
    type: str
    returned: when available
dhcp_config:
    description:
      - The boto2-style DHCP options created, associated or found.
      - This return value has been deprecated and will be removed in a release after
        2026-12-01. Use RV(dhcp_options) instead.
    returned: when available
    type: dict
    contains:
      domain-name-servers:
        description: The IP addresses of up to four domain name servers, or AmazonProvidedDNS.
        returned: when available
        type: list
        sample:
          - 10.0.0.1
          - 10.0.1.1
      domain-name:
        description: The domain name for hosts in the DHCP option sets.
        returned: when available
        type: list
        sample:
          - "my.example.com"
      ntp-servers:
        description: The IP addresses of up to four Network Time Protocol (NTP) servers.
        returned: when available
        type: list
        sample:
          - 10.0.0.1
          - 10.0.1.1
      netbios-name-servers:
        description: The IP addresses of up to four NetBIOS name servers.
        returned: when available
        type: list
        sample:
          - 10.0.0.1
          - 10.0.1.1
      netbios-node-type:
        description: The NetBIOS node type (1, 2, 4, or 8).
        returned: when available
        type: str
        sample: 2
"""

EXAMPLES = r"""
# Completely overrides the VPC DHCP options associated with VPC vpc-123456 and deletes any existing
# DHCP option set that may have been attached to that VPC.
- amazon.aws.ec2_vpc_dhcp_option:
    domain_name: "foo.example.com"
    region: us-east-1
    dns_servers:
      - 10.0.0.1
      - 10.0.1.1
    ntp_servers:
      - 10.0.0.2
      - 10.0.1.2
    netbios_name_servers:
      - 10.0.0.1
      - 10.0.1.1
    netbios_node_type: 2
    vpc_id: vpc-123456
    delete_old: true
    inherit_existing: false

# Ensure the DHCP option set for the VPC has 10.0.0.4 and 10.0.1.4 as the specified DNS servers, but
# keep any other existing settings. Also, keep the old DHCP option set around.
- amazon.aws.ec2_vpc_dhcp_option:
    region: us-east-1
    dns_servers:
      - "{{groups['dns-primary']}}"
      - "{{groups['dns-secondary']}}"
    vpc_id: vpc-123456
    inherit_existing: true
    delete_old: false

## Create a DHCP option set with 4.4.4.4 and 8.8.8.8 as the specified DNS servers, with tags
## but do not assign to a VPC
- amazon.aws.ec2_vpc_dhcp_option:
    region: us-east-1
    dns_servers:
      - 4.4.4.4
      - 8.8.8.8
    tags:
      Name: google servers
      Environment: Test

## Delete a DHCP options set that matches the tags and options specified
- amazon.aws.ec2_vpc_dhcp_option:
    region: us-east-1
    dns_servers:
      - 4.4.4.4
      - 8.8.8.8
    tags:
      Name: google servers
      Environment: Test
    state: absent

## Associate a DHCP options set with a VPC by ID
- amazon.aws.ec2_vpc_dhcp_option:
    region: us-east-1
    dhcp_options_id: dopt-12345678
    vpc_id: vpc-123456
"""

from typing import Any
from typing import Dict
from typing import List
from typing import Optional
from typing import Tuple
from typing import Union

from ansible.module_utils.common.dict_transformations import camel_dict_to_snake_dict

from ansible_collections.amazon.aws.plugins.module_utils.ec2 import AnsibleEC2Error
from ansible_collections.amazon.aws.plugins.module_utils.ec2 import associate_dhcp_options
from ansible_collections.amazon.aws.plugins.module_utils.ec2 import create_dhcp_options
from ansible_collections.amazon.aws.plugins.module_utils.ec2 import delete_dhcp_options
from ansible_collections.amazon.aws.plugins.module_utils.ec2 import describe_dhcp_options
from ansible_collections.amazon.aws.plugins.module_utils.ec2 import describe_vpcs
from ansible_collections.amazon.aws.plugins.module_utils.ec2 import ensure_ec2_tags
from ansible_collections.amazon.aws.plugins.module_utils.ec2 import normalize_ec2_vpc_dhcp_config
from ansible_collections.amazon.aws.plugins.module_utils.modules import AnsibleAWSModule
from ansible_collections.amazon.aws.plugins.module_utils.tagging import ansible_dict_to_boto3_tag_list
from ansible_collections.amazon.aws.plugins.module_utils.tagging import boto3_tag_list_to_ansible_dict
from ansible_collections.amazon.aws.plugins.module_utils.tagging import boto3_tag_specifications


def fetch_dhcp_options_for_vpc(
    client, module: AnsibleAWSModule, vpc_id: str
) -> Tuple[Optional[List[Dict[str, Union[str, List[Dict[str, str]]]]]], Optional[str]]:
    try:
        vpcs = describe_vpcs(client, VpcIds=[vpc_id])
    except AnsibleEC2Error as e:
        module.fail_json_aws(e, msg=f"Unable to describe vpc {vpc_id}")

    if len(vpcs) != 1:
        return None, None
    try:
        dhcp_options = describe_dhcp_options(client, DhcpOptionsIds=[vpcs[0]["DhcpOptionsId"]])
    except AnsibleEC2Error as e:
        module.fail_json_aws(e, msg=f"Unable to describe dhcp option {vpcs[0]['DhcpOptionsId']}")

    if len(dhcp_options) != 1:
        return None, None
    return dhcp_options[0]["DhcpConfigurations"], dhcp_options[0]["DhcpOptionsId"]


def remove_dhcp_options_by_id(client, module: AnsibleAWSModule, dhcp_options_id: str) -> bool:
    changed = False
    # First, check if this dhcp option is associated to any other vpcs
    try:
        associations = describe_vpcs(client, Filters=[{"Name": "dhcp-options-id", "Values": [dhcp_options_id]}])
    except AnsibleEC2Error as e:
        module.fail_json_aws(e, msg=f"Unable to describe VPC associations for dhcp option id {dhcp_options_id}")
    if len(associations) > 0:
        return changed

    changed = True
    if not module.check_mode:
        try:
            changed = delete_dhcp_options(client, dhcp_options_id)
        except AnsibleEC2Error as e:
            module.fail_json_aws(e, msg=f"Unable to delete dhcp option {dhcp_options_id}")

    return changed


def match_dhcp_options(client, module: AnsibleAWSModule, new_config: List[Dict[str, Any]]) -> Optional[str]:
    """
    Returns a DhcpOptionsId if the module parameters match; else None
    Filter by tags, if any are specified
    """
    try:
        all_dhcp_options = describe_dhcp_options(client)
    except AnsibleEC2Error as e:
        module.fail_json_aws(e, msg="Unable to describe dhcp options")

    for dopts in all_dhcp_options:
        if module.params["tags"]:
            # If we were given tags, try to match on them
            boto_tags = ansible_dict_to_boto3_tag_list(module.params["tags"])
            if dopts["DhcpConfigurations"] == new_config and dopts["Tags"] == boto_tags:
                return dopts["DhcpOptionsId"]
        elif dopts["DhcpConfigurations"] == new_config:
            return dopts["DhcpOptionsId"]

    return None


def create_dhcp_config(params: Dict[str, Any]) -> List[Dict[str, Any]]:
    """
    Convert provided parameters into a DhcpConfigurations list that conforms to what the API returns:
    https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_DescribeDhcpOptions.html
        [{'Key': 'domain-name',
         'Values': [{'Value': 'us-west-2.compute.internal'}]},
        {'Key': 'domain-name-servers',
         'Values': [{'Value': 'AmazonProvidedDNS'}]},
         ...],
    """
    new_config = []
    if params["domain_name"] is not None:
        new_config.append({"Key": "domain-name", "Values": [{"Value": params["domain_name"]}]})
    if params["dns_servers"] is not None:
        dns_server_list = []
        for server in params["dns_servers"]:
            dns_server_list.append({"Value": server})
        new_config.append({"Key": "domain-name-servers", "Values": dns_server_list})
    if params["ntp_servers"] is not None:
        ntp_server_list = []
        for server in params["ntp_servers"]:
            ntp_server_list.append({"Value": server})
        new_config.append({"Key": "ntp-servers", "Values": ntp_server_list})
    if params["netbios_name_servers"] is not None:
        netbios_server_list = []
        for server in params["netbios_name_servers"]:
            netbios_server_list.append({"Value": server})
        new_config.append({"Key": "netbios-name-servers", "Values": netbios_server_list})
    if params["netbios_node_type"] is not None:
        new_config.append({"Key": "netbios-node-type", "Values": params["netbios_node_type"]})

    return new_config


def create_dhcp_option_set(
    client, params: Dict[str, Any], new_config: List[Dict[str, Any]], check_mode: bool
) -> Optional[str]:
    """
    A CreateDhcpOptions object looks different than the object we create in create_dhcp_config()
    This is the only place we use it, so create it now
    https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_CreateDhcpOptions.html
    We have to do this after inheriting any existing_config, so we need to start with the object
    that we made in create_dhcp_config().
    normalize_config() gives us the nicest format to work with for this.
    """
    desired_config = normalize_ec2_vpc_dhcp_config(new_config)
    create_config = []
    tags_list = []

    for option in ["domain-name", "domain-name-servers", "ntp-servers", "netbios-name-servers"]:
        if desired_config.get(option):
            create_config.append({"Key": option, "Values": desired_config[option]})
    if desired_config.get("netbios-node-type"):
        # We need to listify this one
        create_config.append({"Key": "netbios-node-type", "Values": [desired_config["netbios-node-type"]]})

    if params.get("tags"):
        tags_list = boto3_tag_specifications(params["tags"], ["dhcp-options"])

    dhcp_options_id = None
    if not check_mode:
        dhcp_options = create_dhcp_options(client, DhcpConfigurations=create_config, TagSpecifications=tags_list)
        dhcp_options_id = dhcp_options["DhcpOptionsId"]
    return dhcp_options_id


def find_opt_index(config: List[Dict[str, Any]], option: str) -> Optional[int]:
    return next((i for i, item in enumerate(config) if item["Key"] == option), None)


def inherit_dhcp_config(
    existing_config: List[Dict[str, Any]], new_config: List[Dict[str, Any]]
) -> Tuple[bool, List[Dict[str, Any]]]:
    """
    Compare two DhcpConfigurations lists and apply existing options to unset parameters

    If there's an existing option config and the new option is not set or it's none,
    inherit the existing config.
    The configs are unordered lists of dicts with non-unique keys, so we have to find
    the right list index for a given config option first.
    """
    changed = False
    for option in ["domain-name", "domain-name-servers", "ntp-servers", "netbios-name-servers", "netbios-node-type"]:
        existing_index = find_opt_index(existing_config, option)
        new_index = find_opt_index(new_config, option)
        # `if existing_index` evaluates to False on index 0, so be very specific and verbose
        if existing_index is not None and new_index is None:
            new_config.append(existing_config[existing_index])
            changed = True

    return changed, new_config


def get_dhcp_options_info(client, dhcp_options_id: Optional[str], check_mode: bool) -> Optional[Dict[str, Any]]:
    # Return boto3-style details, consistent with the _info module

    if check_mode and dhcp_options_id is None:
        # We can't describe without an option id, we might get here when creating a new option set in check_mode
        return None

    dhcp_options = describe_dhcp_options(client, DhcpOptionsIds=[dhcp_options_id])
    dhcp_option_info = None
    if dhcp_options:
        dhcp_option_info = camel_dict_to_snake_dict(
            {
                "DhcpOptionsId": dhcp_options[0]["DhcpOptionsId"],
                "DhcpConfigurations": dhcp_options[0]["DhcpConfigurations"],
                "Tags": boto3_tag_list_to_ansible_dict(dhcp_options[0].get("Tags", [{"Value": "", "Key": "Name"}])),
            },
            ignore_list=["Tags"],
        )
    return dhcp_option_info


def ensure_absent(client, module: AnsibleAWSModule, new_config: List[Dict[str, Any]]) -> None:
    dhcp_options_id = module.params["dhcp_options_id"]
    if not dhcp_options_id:
        # Look up the option id first by matching the supplied options
        dhcp_options_id = match_dhcp_options(client, module, new_config)
    changed = remove_dhcp_options_by_id(client, module, dhcp_options_id)
    module.exit_json(changed=changed, dhcp_options={}, dhcp_config={})


def ensure_present(client, module: AnsibleAWSModule, new_config: List[Dict[str, Any]]) -> None:
    vpc_id = module.params["vpc_id"]
    delete_old = module.params["delete_old"]
    inherit_existing = module.params["inherit_existing"]
    tags = module.params["tags"]
    purge_tags = module.params["purge_tags"]
    dhcp_options_id = module.params["dhcp_options_id"]
    changed = False
    existing_config = None
    existing_id = None

    if not dhcp_options_id:
        # If we were given a vpc_id then we need to look at the configuration on that
        if vpc_id:
            existing_config, existing_id = fetch_dhcp_options_for_vpc(client, module, vpc_id)
            # if we've been asked to inherit existing options, do that now
            if inherit_existing and existing_config:
                changed, new_config = inherit_dhcp_config(existing_config, new_config)
            # Do the vpc's dhcp options already match what we're asked for? if so we are done
            if existing_config:
                if new_config == existing_config:
                    dhcp_options_id = existing_id
                    if tags or purge_tags:
                        changed |= ensure_ec2_tags(
                            client,
                            module,
                            dhcp_options_id,
                            resource_type="dhcp-options",
                            tags=tags,
                            purge_tags=purge_tags,
                        )
                    return_config = normalize_ec2_vpc_dhcp_config(new_config)
                    results = get_dhcp_options_info(client, dhcp_options_id, module.check_mode)
                    module.exit_json(
                        changed=changed,
                        dhcp_options_id=dhcp_options_id,
                        dhcp_options=results,
                        dhcp_config=return_config,
                    )
        # If no vpc_id was given, or the options don't match then look for an existing set using tags
        dhcp_options_id = match_dhcp_options(client, module, new_config)

    else:
        # Now let's cover the case where there are existing options that we were told about by id
        # If a dhcp_options_id was supplied we don't look at options inside, just set tags (if given)
        # Preserve the boto2 module's behaviour of checking if the option set exists first,
        # and return the same error message if it does not
        dhcp_options = describe_dhcp_options(client, DhcpOptionsIds=[dhcp_options_id])
        if not dhcp_options:
            module.fail_json(msg="a dhcp_options_id was supplied, but does not exist")

    if not dhcp_options_id:
        # If we still don't have an options ID, create it
        dhcp_options_id = create_dhcp_option_set(client, module.params, new_config, module.check_mode)
        changed = True
    else:
        if tags or purge_tags:
            changed |= ensure_ec2_tags(
                client, module, dhcp_options_id, resource_type="dhcp-options", tags=tags, purge_tags=purge_tags
            )

    # If we were given a vpc_id, then attach the options we now have to that before we finish
    if vpc_id:
        if module.check_mode:
            changed = True
        else:
            try:
                changed = associate_dhcp_options(client, dhcp_options_id=dhcp_options_id, vpc_id=vpc_id)
            except AnsibleEC2Error as e:
                module.fail_json_aws_error(e)

    if delete_old and existing_id:
        remove_dhcp_options_by_id(client, module, existing_id)

    return_config = normalize_ec2_vpc_dhcp_config(new_config)
    results = get_dhcp_options_info(client, dhcp_options_id, module.check_mode)
    module.exit_json(changed=changed, dhcp_options_id=dhcp_options_id, dhcp_options=results, dhcp_config=return_config)


def main() -> None:
    argument_spec = dict(
        dhcp_options_id=dict(type="str", default=None),
        domain_name=dict(type="str", default=None),
        dns_servers=dict(type="list", elements="str", default=None),
        ntp_servers=dict(type="list", elements="str", default=None),
        netbios_name_servers=dict(type="list", elements="str", default=None),
        netbios_node_type=dict(type="int", default=None),
        vpc_id=dict(type="str", default=None),
        delete_old=dict(type="bool", default=True),
        inherit_existing=dict(type="bool", default=False),
        tags=dict(type="dict", default=None, aliases=["resource_tags"]),
        purge_tags=dict(default=True, type="bool"),
        state=dict(type="str", default="present", choices=["present", "absent"]),
    )

    module = AnsibleAWSModule(argument_spec=argument_spec, check_boto3=False, supports_check_mode=True)

    module.deprecate(
        "The 'dhcp_config' return value is deprecated. Use 'dhcp_options' instead.",
        date="2026-12-01",
        collection_name="amazon.aws",
    )

    state = module.params["state"]
    new_config = create_dhcp_config(module.params)

    client = module.client("ec2")

    try:
        if state == "absent":
            ensure_absent(client, module, new_config)
        else:
            ensure_present(client, module, new_config)
    except AnsibleEC2Error as e:
        module.fail_json_aws_error(e)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2018, Aaron Smith <ajsmith10381@gmail.com>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: config_aggregation_authorization
version_added: 1.0.0
short_description: Manage cross-account AWS Config authorizations
description:
  - Module manages AWS Config aggregation authorizations.
  - Prior to release 5.0.0 this module was called C(community.aws.aws_config_aggregation_authorization).
    The usage did not change.
author:
  - "Aaron Smith (@slapula)"
options:
  state:
    description:
      - Whether the Config rule should be present or absent.
    default: present
    choices: ['present', 'absent']
    type: str
  authorized_account_id:
    description:
      - The 12-digit account ID of the account authorized to aggregate data.
    type: str
    required: true
  authorized_aws_region:
    description:
      - The region authorized to collect aggregated data.
    type: str
    required: true
extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.boto3
"""

EXAMPLES = r"""
- name: Get current account ID
  community.aws.aws_caller_info:
  register: whoami
- community.aws.config_aggregation_authorization:
    state: present
    authorized_account_id: '{{ whoami.account }}'
    authorized_aws_region: us-east-1
"""

RETURN = r"""#"""

try:
    import botocore
except ImportError:
    pass  # handled by AnsibleAWSModule

from ansible_collections.amazon.aws.plugins.module_utils.retries import AWSRetry

from ansible_collections.community.aws.plugins.module_utils.modules import AnsibleCommunityAWSModule as AnsibleAWSModule


def resource_exists(client, module, params):
    try:
        current_authorizations = client.describe_aggregation_authorizations()["AggregationAuthorizations"]
        authorization_exists = next(
            (item for item in current_authorizations if item["AuthorizedAccountId"] == params["AuthorizedAccountId"]),
            None,
        )
        if authorization_exists:
            return True
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError):
        return False


def create_resource(client, module, params, result):
    try:
        client.put_aggregation_authorization(
            AuthorizedAccountId=params["AuthorizedAccountId"],
            AuthorizedAwsRegion=params["AuthorizedAwsRegion"],
        )
        result["changed"] = True
        return result
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
        module.fail_json_aws(e, msg="Couldn't create AWS Aggregation authorization")


def update_resource(client, module, params, result):
    current_authorizations = client.describe_aggregation_authorizations()["AggregationAuthorizations"]
    current_params = next(
        (item for item in current_authorizations if item["AuthorizedAccountId"] == params["AuthorizedAccountId"]),
        None,
    )

    del current_params["AggregationAuthorizationArn"]
    del current_params["CreationTime"]

    if params != current_params:
        try:
            client.put_aggregation_authorization(
                AuthorizedAccountId=params["AuthorizedAccountId"],
                AuthorizedAwsRegion=params["AuthorizedAwsRegion"],
            )
            result["changed"] = True
            return result
        except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
            module.fail_json_aws(e, msg="Couldn't create AWS Aggregation authorization")


def delete_resource(client, module, params, result):
    try:
        client.delete_aggregation_authorization(
            AuthorizedAccountId=params["AuthorizedAccountId"],
            AuthorizedAwsRegion=params["AuthorizedAwsRegion"],
        )
        result["changed"] = True
        return result
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
        module.fail_json_aws(e, msg="Couldn't delete AWS Aggregation authorization")


def main():
    module = AnsibleAWSModule(
        argument_spec={
            "state": dict(type="str", choices=["present", "absent"], default="present"),
            "authorized_account_id": dict(type="str", required=True),
            "authorized_aws_region": dict(type="str", required=True),
        },
        supports_check_mode=False,
    )

    result = {"changed": False}

    params = {
        "AuthorizedAccountId": module.params.get("authorized_account_id"),
        "AuthorizedAwsRegion": module.params.get("authorized_aws_region"),
    }

    client = module.client("config", retry_decorator=AWSRetry.jittered_backoff())
    resource_status = resource_exists(client, module, params)

    if module.params.get("state") == "present":
        if not resource_status:
            create_resource(client, module, params, result)
        else:
            update_resource(client, module, params, result)

    if module.params.get("state") == "absent":
        if resource_status:
            delete_resource(client, module, params, result)

    module.exit_json(changed=result["changed"])


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Ansible module to manage Check Point Firewall (c) 2019
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {
    "metadata_version": "1.1",
    "status": ["preview"],
    "supported_by": "community",
}

DOCUMENTATION = """
---
module: cp_mgmt_access_rules
short_description: Manages access-rules objects on Check Point over Web Services API
description:
  - Manages access-rules objects on Check Point devices including creating, updating and removing objects.
  - All operations are performed over Web Services API.
  - Available from R80 management version.
version_added: "2.2.0"
author: "Shiran Golzar (@chkp-shirango)"
options:
  layer:
    description:
      - Layer that the rule belongs to identified by the name or UID.
    type: str
    required: true
  rules:
    description:
      - List of rules.
    type: list
    elements: dict
    required: true
    suboptions:
      name:
        description:
          - Object name.
        type: str
        required: True
      action:
        description:
          - a "Accept", "Drop", "Ask", "Inform", "Reject", "User Auth", "Client Auth", "Apply Layer".
        type: str
      action_settings:
        description:
          - Action settings.
        type: dict
        suboptions:
          enable_identity_captive_portal:
            description:
              - N/A
            type: bool
          limit:
            description:
              - N/A
            type: str
      content:
        description:
          - List of processed file types that this rule applies on.
        type: list
        elements: dict
      content_direction:
        description:
          - On which direction the file types processing is applied.
        type: str
        choices: ['any', 'up', 'down']
      content_negate:
        description:
          - True if negate is set for data.
        type: bool
      custom_fields:
        description:
          - Custom fields.
        type: dict
        suboptions:
          field_1:
            description:
              - First custom field.
            type: str
          field_2:
            description:
              - Second custom field.
            type: str
          field_3:
            description:
              - Third custom field.
            type: str
      destination:
        description:
          - Collection of Network objects identified by the name or UID.
        type: list
        elements: str
      destination_negate:
        description:
          - True if negate is set for destination.
        type: bool
      enabled:
        description:
          - Enable/Disable the rule.
        type: bool
      inline_layer:
        description:
          - Inline Layer identified by the name or UID. Relevant only if "Action" was set to "Apply Layer".
        type: str
      install_on:
        description:
          - Which Gateways identified by the name or UID to install the policy on.
        type: list
        elements: str
      service:
        description:
          - Collection of Network objects identified by the name or UID.
        type: list
        elements: str
      service_negate:
        description:
          - True if negate is set for service.
        type: bool
      source:
        description:
          - Collection of Network objects identified by the name or UID.
        type: list
        elements: str
      source_negate:
        description:
          - True if negate is set for source.
        type: bool
      time:
        description:
          - List of time objects. For example, "Weekend", "Off-Work", "Every-Day".
        type: list
        elements: str
      track:
        description:
          - Track Settings.
        type: dict
        suboptions:
          accounting:
            description:
              - Turns accounting for track on and off.
            type: bool
          alert:
            description:
              - Type of alert for the track.
            type: str
            choices: ['none', 'alert', 'snmp', 'mail', 'user alert 1', 'user alert 2', 'user alert 3']
          enable_firewall_session:
            description:
              - Determine whether to generate session log to firewall only connections.
            type: bool
          per_connection:
            description:
              - Determines whether to perform the log per connection.
            type: bool
          per_session:
            description:
              - Determines whether to perform the log per session.
            type: bool
          type:
            description:
              - a "Log", "Extended Log", "Detailed  Log", "None".
            type: str
      user_check:
        description:
          - User check settings.
        type: dict
        suboptions:
          confirm:
            description:
              - N/A
            type: str
            choices: ['per rule', 'per category', 'per application/site', 'per data type']
          custom_frequency:
            description:
              - N/A
            type: dict
            suboptions:
              every:
                description:
                  - N/A
                type: int
              unit:
                description:
                  - N/A
                type: str
                choices: ['hours', 'days', 'weeks', 'months']
          frequency:
            description:
              - N/A
            type: str
            choices: ['once a day', 'once a week', 'once a month', 'custom frequency...']
          interaction:
            description:
              - N/A
            type: str
      vpn_list:
        description:
          - Communities or Directional.
        type: list
        elements: dict
        suboptions:
          community:
            description:
              - List of community name or UID.
            type: list
            elements: str
          directional:
            description:
              - Communities directional match condition.
            type: list
            elements: dict
            suboptions:
              from:
                description:
                  - From community name or UID.
                type: str
              to:
                description:
                  - To community name or UID.
                type: str
      vpn:
        description:
          - Any or All_GwToGw.
        type: str
        choices: ['Any', 'All_GwToGw']
      comments:
        description:
          - Comments string.
        type: str
      details_level:
        description:
          - The level of detail for some of the fields in the response can vary from showing only the UID value of the object to a fully detailed
            representation of the object.
        type: str
        choices: ['uid', 'standard', 'full']
      ignore_warnings:
        description:
          - Apply changes ignoring warnings.
        type: bool
      ignore_errors:
        description:
          - Apply changes ignoring errors. You won't be able to publish such a changes. If ignore-warnings flag was omitted - warnings will also be ignored.
        type: bool
      state:
        description:
          - State of the access rule (present or absent). Defaults to present.
        type: str
        default: present
        choices:
          - 'present'
          - 'absent'
  details_level:
    description:
      - The level of detail for some of the fields in the response can vary from showing only the UID value of the object to a fully detailed
        representation of the object.
    type: str
    choices: ['uid', 'standard', 'full']
extends_documentation_fragment: check_point.mgmt.checkpoint_objects_action_module
"""

EXAMPLES = """
- name: add-access-rules
  cp_mgmt_access_rules:
    rules:
      - name: Rule 1
        service:
          - SMTP
          - AOL
        state: present
      - name: Rule 2
        service:
          - SMTP
        state: present
    layer: Network
    auto_publish_session: true
"""

RETURN = """
cp_mgmt_access_rules:
  description: The checkpoint object created or updated.
  returned: always, except when deleting the object.
  type: dict
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.check_point.mgmt.plugins.module_utils.checkpoint import (
    checkpoint_argument_spec_for_action_module,
)


def main():
    argument_spec = dict(
        rules=dict(
            type="list",
            required=True,
            elements="dict",
            options=dict(
                name=dict(type="str", required=True),
                action=dict(type="str"),
                action_settings=dict(
                    type="dict",
                    options=dict(
                        enable_identity_captive_portal=dict(type="bool"),
                        limit=dict(type="str"),
                    ),
                ),
                content=dict(type="list", elements="dict"),
                content_direction=dict(
                    type="str", choices=["any", "up", "down"]
                ),
                content_negate=dict(type="bool"),
                custom_fields=dict(
                    type="dict",
                    options=dict(
                        field_1=dict(type="str"),
                        field_2=dict(type="str"),
                        field_3=dict(type="str"),
                    ),
                ),
                destination=dict(type="list", elements="str"),
                destination_negate=dict(type="bool"),
                enabled=dict(type="bool"),
                inline_layer=dict(type="str"),
                install_on=dict(type="list", elements="str"),
                service=dict(type="list", elements="str"),
                service_negate=dict(type="bool"),
                source=dict(type="list", elements="str"),
                source_negate=dict(type="bool"),
                time=dict(type="list", elements="str"),
                track=dict(
                    type="dict",
                    options=dict(
                        accounting=dict(type="bool"),
                        alert=dict(
                            type="str",
                            choices=[
                                "none",
                                "alert",
                                "snmp",
                                "mail",
                                "user alert 1",
                                "user alert 2",
                                "user alert 3",
                            ],
                        ),
                        enable_firewall_session=dict(type="bool"),
                        per_connection=dict(type="bool"),
                        per_session=dict(type="bool"),
                        type=dict(type="str"),
                    ),
                ),
                user_check=dict(
                    type="dict",
                    options=dict(
                        confirm=dict(
                            type="str",
                            choices=[
                                "per rule",
                                "per category",
                                "per application/site",
                                "per data type",
                            ],
                        ),
                        custom_frequency=dict(
                            type="dict",
                            options=dict(
                                every=dict(type="int"),
                                unit=dict(
                                    type="str",
                                    choices=[
                                        "hours",
                                        "days",
                                        "weeks",
                                        "months",
                                    ],
                                ),
                            ),
                        ),
                        frequency=dict(
                            type="str",
                            choices=[
                                "once a day",
                                "once a week",
                                "once a month",
                                "custom frequency...",
                            ],
                        ),
                        interaction=dict(type="str"),
                    ),
                ),
                vpn_list=dict(
                    type="list",
                    elements="dict",
                    options=dict(
                        community=dict(type="list", elements="str"),
                        directional=dict(
                            type="list",
                            elements="dict",
                            options=dict(to=dict(type="str")),
                        ),
                    ),
                ),
                vpn=dict(type="str", choices=["Any", "All_GwToGw"]),
                comments=dict(type="str"),
                details_level=dict(
                    type="str", choices=["uid", "standard", "full"]
                ),
                ignore_warnings=dict(type="bool"),
                ignore_errors=dict(type="bool"),
                state=dict(
                    type="str",
                    choices=["present", "absent"],
                    default="present",
                ),
            ),
        ),
        layer=dict(type="str", required=True),
        details_level=dict(type="str", choices=["uid", "standard", "full"]),
    )

    argument_spec["rules"]["options"]["vpn_list"]["options"]["directional"][
        "options"
    ]["from"] = dict(type="str")
    argument_spec.update(checkpoint_argument_spec_for_action_module)

    module = AnsibleModule(argument_spec=argument_spec)

    module.exit_json()


if __name__ == "__main__":
    main()



#!/usr/bin/python

# Copyright (c) 2016, James Hogarth <james.hogarth@gmail.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
author: James Hogarth (@hogarthj)
module: jenkins_script
short_description: Executes a groovy script in the jenkins instance
description:
  - The C(jenkins_script) module takes a script plus a dict of values to use within the script and returns the result of the
    script being run.
extends_documentation_fragment:
  - community.general._attributes

attributes:
  check_mode:
    support: none
  diff_mode:
    support: none

options:
  script:
    type: str
    description:
      - The groovy script to be executed. This gets passed as a string Template if args is defined.
    required: true
  url:
    type: str
    description:
      - The jenkins server to execute the script against. The default is a local jenkins instance that is not being proxied
        through a webserver.
    default: http://localhost:8080
  validate_certs:
    description:
      - If set to V(false), the SSL certificates are not validated. This should only set to V(false) used on personally controlled
        sites using self-signed certificates as it avoids verifying the source site.
    type: bool
    default: true
  user:
    type: str
    description:
      - The username to connect to the jenkins server with.
  password:
    type: str
    description:
      - The password to connect to the jenkins server with.
  timeout:
    type: int
    description:
      - The request timeout in seconds.
    default: 10
  args:
    type: dict
    description:
      - A dict of key-value pairs used in formatting the script using string.Template (see https://docs.python.org/2/library/string.html#template-strings).
notes:
  - Since the script can do anything this does not report on changes. Knowing the script is being run it is important to set
    C(changed_when) for the ansible output to be clear on any alterations made.
"""

EXAMPLES = r"""
- name: Obtaining a list of plugins
  community.general.jenkins_script:
    script: 'println(Jenkins.instance.pluginManager.plugins)'
    user: admin
    password: admin

- name: Setting master using a variable to hold a more complicate script
  ansible.builtin.set_fact:
    setmaster_mode: |
      import jenkins.model.*
      instance = Jenkins.getInstance()
      instance.setMode(${jenkins_mode})
      instance.save()

- name: Use the variable as the script
  community.general.jenkins_script:
    script: "{{ setmaster_mode }}"
    args:
      jenkins_mode: Node.Mode.EXCLUSIVE

- name: Interacting with an untrusted HTTPS connection
  community.general.jenkins_script:
    script: "println(Jenkins.instance.pluginManager.plugins)"
    user: admin
    password: admin
    url: https://localhost
    validate_certs: false # only do this when you trust the network!
"""

RETURN = r"""
output:
  description: Result of script.
  returned: success
  type: str
  sample: 'Result: true'
"""

import json
from http import cookiejar
from string import Template
from urllib.parse import urlencode

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.common.text.converters import to_native
from ansible.module_utils.urls import fetch_url


def is_csrf_protection_enabled(module):
    resp, info = fetch_url(module, f"{module.params['url']}/api/json", timeout=module.params["timeout"], method="GET")
    if info["status"] != 200:
        module.fail_json(msg=f"HTTP error {info['status']} {info['msg']}", output="")

    return json.loads(resp.read()).get("useCrumbs", False)


def get_crumb(module, cookies):
    resp, info = fetch_url(
        module,
        f"{module.params['url']}/crumbIssuer/api/json",
        method="GET",
        timeout=module.params["timeout"],
        cookies=cookies,
    )
    if info["status"] != 200:
        module.fail_json(msg=f"HTTP error {info['status']} {info['msg']}", output="")

    return json.loads(resp.read())


def main():
    module = AnsibleModule(
        argument_spec=dict(
            script=dict(required=True, type="str"),
            url=dict(type="str", default="http://localhost:8080"),
            validate_certs=dict(type="bool", default=True),
            user=dict(type="str"),
            password=dict(no_log=True, type="str"),
            timeout=dict(type="int", default=10),
            args=dict(type="dict"),
        )
    )

    if module.params["user"] is not None:
        if module.params["password"] is None:
            module.fail_json(msg="password required when user provided", output="")
        module.params["url_username"] = module.params["user"]
        module.params["url_password"] = module.params["password"]
        module.params["force_basic_auth"] = True

    if module.params["args"] is not None:
        try:
            script_contents = Template(module.params["script"]).substitute(module.params["args"])
        except KeyError as err:
            module.fail_json(msg=f"Error with templating variable: {err}", output="")
    else:
        script_contents = module.params["script"]

    headers = {}
    cookies = None
    if is_csrf_protection_enabled(module):
        cookies = cookiejar.LWPCookieJar()
        crumb = get_crumb(module, cookies)
        headers = {crumb["crumbRequestField"]: crumb["crumb"]}

    resp, info = fetch_url(
        module,
        f"{module.params['url']}/scriptText",
        data=urlencode({"script": script_contents}),
        headers=headers,
        method="POST",
        timeout=module.params["timeout"],
        cookies=cookies,
    )

    if info["status"] != 200:
        module.fail_json(msg=f"HTTP error {info['status']} {info['msg']}", output="")

    result = to_native(resp.read())

    if "Exception:" in result and "at java.lang.Thread" in result:
        module.fail_json(msg=f"script failed with stacktrace:\n {result}", output="")

    module.exit_json(
        output=result,
    )


if __name__ == "__main__":
    main()



#!/usr/bin/python

# Copyright (c) 2016, Peter Sagerson <psagers@ignorare.net>
# Copyright (c) 2020, Sebastian Pfahl <eryx@gmx.net>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: ldap_search
version_added: '0.2.0'
short_description: Search for entries in a LDAP server
description:
  - Return the results of an LDAP search.
author:
  - Sebastian Pfahl (@eryx12o45)
requirements:
  - python-ldap
attributes:
  check_mode:
    support: full
  diff_mode:
    support: none
options:
  dn:
    required: true
    type: str
    description:
      - The LDAP DN to search in.
  scope:
    choices: [base, onelevel, subordinate, children]
    default: base
    type: str
    description:
      - The LDAP scope to use.
      - V(subordinate) requires the LDAPv3 subordinate feature extension.
      - V(children) is equivalent to a "subtree" scope.
  filter:
    default: '(objectClass=*)'
    type: str
    description:
      - Used for filtering the LDAP search result.
  attrs:
    type: list
    elements: str
    description:
      - A list of attributes for limiting the result. Use an actual list or a comma-separated string.
  schema:
    default: false
    type: bool
    description:
      - Set to V(true) to return the full attribute schema of entries, not their attribute values. Overrides O(attrs) when
        provided.
  page_size:
    default: 0
    type: int
    description:
      - The page size when performing a simple paged result search (RFC 2696). This setting can be tuned to reduce issues
        with timeouts and server limits.
      - Setting the page size to V(0) (default) disables paged searching.
    version_added: 7.1.0
  base64_attributes:
    description:
      - If provided, all attribute values returned that are listed in this option are Base64 encoded.
      - If the special value V(*) appears in this list, all attributes are Base64 encoded.
      - All other attribute values are converted to UTF-8 strings. If they contain binary data, please note that invalid UTF-8
        bytes are omitted.
    type: list
    elements: str
    version_added: 7.0.0
extends_documentation_fragment:
  - community.general._ldap.documentation
  - community.general._attributes
"""

EXAMPLES = r"""
- name: Return all entries within the 'groups' organizational unit.
  community.general.ldap_search:
    dn: "ou=groups,dc=example,dc=com"
  register: ldap_groups

- name: Return GIDs for all groups
  community.general.ldap_search:
    dn: "ou=groups,dc=example,dc=com"
    scope: "onelevel"
    attrs:
      - "gidNumber"
  register: ldap_group_gids
"""

# @FIXME RV 'results' is meant to be used when 'loop:' was used with the module.
RESULTS = r"""
results:
  description:
    - For every entry found, one dictionary is returned.
    - Every dictionary contains a key C(dn) with the entry's DN as a value.
    - Every attribute of the entry found is added to the dictionary. If the key has precisely one value, that value is taken
      directly, otherwise the key's value is a list.
    - Note that all values (for single-element lists) and list elements (for multi-valued lists) are UTF-8 strings. Some might
      contain Base64-encoded binary data; which ones is determined by the O(base64_attributes) option.
  type: list
  elements: dict
"""

import base64
import traceback

from ansible.module_utils.basic import AnsibleModule, missing_required_lib
from ansible.module_utils.common.text.converters import to_bytes, to_text

from ansible_collections.community.general.plugins.module_utils._ldap import (
    LdapGeneric,
    gen_specs,
    ldap_required_together,
)

LDAP_IMP_ERR = None
try:
    import ldap

    HAS_LDAP = True
except ImportError:
    LDAP_IMP_ERR = traceback.format_exc()
    HAS_LDAP = False


def main():
    module = AnsibleModule(
        argument_spec=gen_specs(
            dn=dict(type="str", required=True),
            scope=dict(type="str", default="base", choices=["base", "onelevel", "subordinate", "children"]),
            filter=dict(type="str", default="(objectClass=*)"),
            attrs=dict(type="list", elements="str"),
            schema=dict(type="bool", default=False),
            page_size=dict(type="int", default=0),
            base64_attributes=dict(type="list", elements="str"),
        ),
        supports_check_mode=True,
        required_together=ldap_required_together(),
    )

    if not HAS_LDAP:
        module.fail_json(msg=missing_required_lib("python-ldap"), exception=LDAP_IMP_ERR)

    try:
        LdapSearch(module).main()
    except Exception as exception:
        module.fail_json(msg="Attribute action failed.", details=f"{exception}")


def _normalize_string(val, convert_to_base64):
    if isinstance(val, (str, bytes)):
        if isinstance(val, str):
            val = to_bytes(val, encoding="utf-8")
        if convert_to_base64:
            val = to_text(base64.b64encode(val))
        else:
            # See https://github.com/ansible/ansible/issues/80258#issuecomment-1477038952 for details.
            # We want to make sure that all strings are properly UTF-8 encoded, even if they were not,
            # or happened to be byte strings.
            val = to_text(val, "utf-8", errors="replace")
            # See also https://github.com/ansible-collections/community.general/issues/5704.
    return val


def _extract_entry(dn, attrs, base64_attributes):
    extracted = {"dn": dn}
    for attr, val in list(attrs.items()):
        convert_to_base64 = "*" in base64_attributes or attr in base64_attributes
        if len(val) == 1:
            extracted[attr] = _normalize_string(val[0], convert_to_base64)
        else:
            extracted[attr] = [_normalize_string(v, convert_to_base64) for v in val]
    return extracted


class LdapSearch(LdapGeneric):
    def __init__(self, module):
        LdapGeneric.__init__(self, module)

        self.filterstr = self.module.params["filter"]
        self.attrlist = []
        self.page_size = self.module.params["page_size"]
        self._load_scope()
        self._load_attrs()
        self._load_schema()
        self._base64_attributes = set(self.module.params["base64_attributes"] or [])

    def _load_schema(self):
        self.schema = self.module.params["schema"]
        if self.schema:
            self.attrsonly = 1
        else:
            self.attrsonly = 0

    def _load_scope(self):
        spec = dict(
            base=ldap.SCOPE_BASE,
            onelevel=ldap.SCOPE_ONELEVEL,
            subordinate=ldap.SCOPE_SUBORDINATE,
            children=ldap.SCOPE_SUBTREE,
        )
        self.scope = spec[self.module.params["scope"]]

    def _load_attrs(self):
        self.attrlist = self.module.params["attrs"] or None

    def main(self):
        results = self.perform_search()
        self.module.exit_json(changed=False, results=results)

    def perform_search(self):
        ldap_entries = []
        controls = []
        if self.page_size > 0:
            controls.append(ldap.controls.libldap.SimplePagedResultsControl(True, size=self.page_size, cookie=""))
        try:
            while True:
                response = self.connection.search_ext(
                    self.dn,
                    self.scope,
                    filterstr=self.filterstr,
                    attrlist=self.attrlist,
                    attrsonly=self.attrsonly,
                    serverctrls=controls,
                )
                rtype, results, rmsgid, serverctrls = self.connection.result3(response)
                for result in results:
                    if isinstance(result[1], dict):
                        if self.schema:
                            ldap_entries.append(dict(dn=result[0], attrs=list(result[1].keys())))
                        else:
                            ldap_entries.append(_extract_entry(result[0], result[1], self._base64_attributes))
                cookies = [
                    c.cookie
                    for c in serverctrls
                    if c.controlType == ldap.controls.libldap.SimplePagedResultsControl.controlType
                ]
                if self.page_size > 0 and cookies and cookies[0]:
                    controls[0].cookie = cookies[0]
                else:
                    return ldap_entries
        except ldap.NO_SUCH_OBJECT:
            self.module.fail_json(msg=f"Base not found: {self.dn}")


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: ecs_ecr
version_added: 1.0.0
short_description: Manage Elastic Container Registry repositories
description:
    - Manage Elastic Container Registry repositories.
options:
    name:
        description:
            - The name of the repository.
        required: true
        type: str
    registry_id:
        description:
            - AWS account id associated with the registry.
            - If not specified, the default registry is assumed.
        required: false
        type: str
    policy:
        description:
            - JSON or dict that represents the new policy.
        required: false
        type: json
    force_absent:
        description:
            - If I(force_absent=true), the repository will be removed, even if images are present.
        required: false
        default: false
        type: bool
        version_added: 4.1.0
    force_set_policy:
        description:
            - If I(force_set_policy=false), it prevents setting a policy that would prevent you from
              setting another policy in the future.
        required: false
        default: false
        type: bool
    purge_policy:
        description:
            - If C(true), remove the policy from the repository.
            - Defaults to C(false).
        required: false
        type: bool
    image_tag_mutability:
        description:
            - Configure whether repository should be mutable (ie. an already existing tag can be overwritten) or not.
        required: false
        choices: [mutable, immutable]
        default: 'mutable'
        type: str
    lifecycle_policy:
        description:
            - JSON or dict that represents the new lifecycle policy.
        required: false
        type: json
    purge_lifecycle_policy:
        description:
            - if C(true), remove the lifecycle policy from the repository.
            - Defaults to C(false).
        required: false
        type: bool
    state:
        description:
            - Create or destroy the repository.
        required: false
        choices: [present, absent]
        default: 'present'
        type: str
    scan_on_push:
        description:
            - if C(true), images are scanned for known vulnerabilities after being pushed to the repository.
        required: false
        default: false
        type: bool
        version_added: 1.3.0
    encryption_configuration:
        description:
            - The encryption configuration for the repository.
        required: false
        suboptions:
            encryption_type:
                description:
                    - The encryption type to use.
                choices: [AES256, KMS]
                default: 'AES256'
                type: str
            kms_key:
                description:
                    - If I(encryption_type=KMS), specify the KMS key to use for encryption.
                    - The alias, key ID, or full ARN of the KMS key can be specified.
                type: str
        type: dict
        version_added: 5.2.0
author:
    - David M. Lee (@leedm777)
extends_documentation_fragment:
    - amazon.aws.common.modules
    - amazon.aws.region.modules
    - amazon.aws.boto3
"""

EXAMPLES = r"""
# If the repository does not exist, it is created. If it does exist, would not
# affect any policies already on it.
- name: ecr-repo
  community.aws.ecs_ecr:
    name: super/cool

- name: destroy-ecr-repo
  community.aws.ecs_ecr:
    name: old/busted
    state: absent

- name: Cross account ecr-repo
  community.aws.ecs_ecr:
    registry_id: 123456789012
    name: cross/account

- name: set-policy as object
  community.aws.ecs_ecr:
    name: needs-policy-object
    policy:
      Version: '2008-10-17'
      Statement:
        - Sid: read-only
          Effect: Allow
          Principal:
            AWS: '{{ read_only_arn }}'
          Action:
            - ecr:GetDownloadUrlForLayer
            - ecr:BatchGetImage
            - ecr:BatchCheckLayerAvailability

- name: set-policy as string
  community.aws.ecs_ecr:
    name: needs-policy-string
    policy: "{{ lookup('template', 'policy.json.j2') }}"

- name: delete-policy
  community.aws.ecs_ecr:
    name: needs-no-policy
    purge_policy: true

- name: create immutable ecr-repo
  community.aws.ecs_ecr:
    name: super/cool
    image_tag_mutability: immutable

- name: set-lifecycle-policy
  community.aws.ecs_ecr:
    name: needs-lifecycle-policy
    scan_on_push: true
    lifecycle_policy:
      rules:
        - rulePriority: 1
          description: new policy
          selection:
            tagStatus: untagged
            countType: sinceImagePushed
            countUnit: days
            countNumber: 365
          action:
            type: expire

- name: purge-lifecycle-policy
  community.aws.ecs_ecr:
    name: needs-no-lifecycle-policy
    purge_lifecycle_policy: true

- name: set-encryption-configuration
  community.aws.ecs_ecr:
    name: uses-custom-kms-key
    encryption_configuration:
      encryption_type: KMS
      kms_key: custom-kms-key-alias
"""

RETURN = r"""
state:
    type: str
    description: The asserted state of the repository (present, absent)
    returned: always
created:
    type: bool
    description: If true, the repository was created
    returned: always
name:
    type: str
    description: The name of the repository
    returned: I(state=absent)
policy:
    type: dict
    description: The existing, created or updated repository policy.
    returned: I(state=present)
    version_added: 4.0.0
repository:
    type: dict
    description: The created or updated repository
    returned: I(state=present)
    sample:
        createdAt: '2017-01-17T08:41:32-06:00'
        registryId: '123456789012'
        repositoryArn: arn:aws:ecr:us-east-1:123456789012:repository/ecr-test-1484664090
        repositoryName: ecr-test-1484664090
        repositoryUri: 123456789012.dkr.ecr.us-east-1.amazonaws.com/ecr-test-1484664090
"""

import json
import traceback

try:
    import botocore
except ImportError:
    pass  # Handled by AnsibleAWSModule

from ansible.module_utils.common.dict_transformations import snake_dict_to_camel_dict

from ansible_collections.amazon.aws.plugins.module_utils.botocore import boto_exception
from ansible_collections.amazon.aws.plugins.module_utils.botocore import is_boto3_error_code
from ansible_collections.amazon.aws.plugins.module_utils.policy import compare_policies

from ansible_collections.community.aws.plugins.module_utils.modules import AnsibleCommunityAWSModule as AnsibleAWSModule


def build_kwargs(registry_id):
    """
    Builds a kwargs dict which may contain the optional registryId.

    :param registry_id: Optional string containing the registryId.
    :return: kwargs dict with registryId, if given
    """
    if not registry_id:
        return dict()
    else:
        return dict(registryId=registry_id)


class EcsEcr:
    def __init__(self, module):
        self.ecr = module.client("ecr")
        self.sts = module.client("sts")
        self.check_mode = module.check_mode
        self.changed = False
        self.skipped = False

    def get_repository(self, registry_id, name):
        try:
            res = self.ecr.describe_repositories(repositoryNames=[name], **build_kwargs(registry_id))
            repos = res.get("repositories")
            return repos and repos[0]
        except is_boto3_error_code("RepositoryNotFoundException"):
            return None

    def get_repository_policy(self, registry_id, name):
        try:
            res = self.ecr.get_repository_policy(repositoryName=name, **build_kwargs(registry_id))
            text = res.get("policyText")
            return text and json.loads(text)
        except is_boto3_error_code(["RepositoryNotFoundException", "RepositoryPolicyNotFoundException"]):
            return None

    def create_repository(self, registry_id, name, image_tag_mutability, encryption_configuration):
        if registry_id:
            default_registry_id = self.sts.get_caller_identity().get("Account")
            if registry_id != default_registry_id:
                raise Exception(
                    f"Cannot create repository in registry {registry_id}.  Would be created in {default_registry_id} instead."
                )

        if encryption_configuration is None:
            encryption_configuration = dict(encryptionType="AES256")

        if not self.check_mode:
            repo = self.ecr.create_repository(
                repositoryName=name,
                imageTagMutability=image_tag_mutability,
                encryptionConfiguration=encryption_configuration,
            ).get("repository")
            self.changed = True
            return repo
        else:
            self.skipped = True
            return dict(repositoryName=name)

    def set_repository_policy(self, registry_id, name, policy_text, force):
        if not self.check_mode:
            policy = self.ecr.set_repository_policy(
                repositoryName=name, policyText=policy_text, force=force, **build_kwargs(registry_id)
            )
            self.changed = True
            return policy
        else:
            self.skipped = True
            if self.get_repository(registry_id, name) is None:
                printable = name
                if registry_id:
                    printable = f"{registry_id}:{name}"
                raise Exception(f"could not find repository {printable}")
            return

    def delete_repository(self, registry_id, name, force):
        if not self.check_mode:
            repo = self.ecr.delete_repository(repositoryName=name, force=force, **build_kwargs(registry_id))
            self.changed = True
            return repo
        else:
            repo = self.get_repository(registry_id, name)
            if repo:
                self.skipped = True
                return repo
            return None

    def delete_repository_policy(self, registry_id, name):
        if not self.check_mode:
            policy = self.ecr.delete_repository_policy(repositoryName=name, **build_kwargs(registry_id))
            self.changed = True
            return policy
        else:
            policy = self.get_repository_policy(registry_id, name)
            if policy:
                self.skipped = True
                return policy
            return None

    def put_image_tag_mutability(self, registry_id, name, new_mutability_configuration):
        repo = self.get_repository(registry_id, name)
        current_mutability_configuration = repo.get("imageTagMutability")

        if current_mutability_configuration != new_mutability_configuration:
            if not self.check_mode:
                self.ecr.put_image_tag_mutability(
                    repositoryName=name, imageTagMutability=new_mutability_configuration, **build_kwargs(registry_id)
                )
            else:
                self.skipped = True
            self.changed = True

        repo["imageTagMutability"] = new_mutability_configuration
        return repo

    def get_lifecycle_policy(self, registry_id, name):
        try:
            res = self.ecr.get_lifecycle_policy(repositoryName=name, **build_kwargs(registry_id))
            text = res.get("lifecyclePolicyText")
            return text and json.loads(text)
        except is_boto3_error_code(["LifecyclePolicyNotFoundException", "RepositoryNotFoundException"]):
            return None

    def put_lifecycle_policy(self, registry_id, name, policy_text):
        if not self.check_mode:
            policy = self.ecr.put_lifecycle_policy(
                repositoryName=name, lifecyclePolicyText=policy_text, **build_kwargs(registry_id)
            )
            self.changed = True
            return policy
        else:
            self.skipped = True
            if self.get_repository(registry_id, name) is None:
                printable = name
                if registry_id:
                    printable = f"{registry_id}:{name}"
                raise Exception(f"could not find repository {printable}")
            return

    def purge_lifecycle_policy(self, registry_id, name):
        if not self.check_mode:
            policy = self.ecr.delete_lifecycle_policy(repositoryName=name, **build_kwargs(registry_id))
            self.changed = True
            return policy
        else:
            policy = self.get_lifecycle_policy(registry_id, name)
            if policy:
                self.skipped = True
                return policy
            return None

    def put_image_scanning_configuration(self, registry_id, name, scan_on_push):
        if not self.check_mode:
            if registry_id:
                scan = self.ecr.put_image_scanning_configuration(
                    registryId=registry_id, repositoryName=name, imageScanningConfiguration={"scanOnPush": scan_on_push}
                )
            else:
                scan = self.ecr.put_image_scanning_configuration(
                    repositoryName=name, imageScanningConfiguration={"scanOnPush": scan_on_push}
                )
            self.changed = True
            return scan
        else:
            self.skipped = True
            return None


def sort_lists_of_strings(policy):
    for statement_index in range(0, len(policy.get("Statement", []))):
        for key in policy["Statement"][statement_index]:
            value = policy["Statement"][statement_index][key]
            if isinstance(value, list) and all(isinstance(item, str) for item in value):
                policy["Statement"][statement_index][key] = sorted(value)
    return policy


def run(ecr, params):
    # type: (EcsEcr, dict, int) -> Tuple[bool, dict]
    result = {}
    try:
        name = params["name"]
        state = params["state"]
        policy_text = params["policy"]
        purge_policy = params["purge_policy"]
        force_absent = params["force_absent"]
        registry_id = params["registry_id"]
        force_set_policy = params["force_set_policy"]
        image_tag_mutability = params["image_tag_mutability"].upper()
        lifecycle_policy_text = params["lifecycle_policy"]
        purge_lifecycle_policy = params["purge_lifecycle_policy"]
        scan_on_push = params["scan_on_push"]
        encryption_configuration = snake_dict_to_camel_dict(params["encryption_configuration"])

        # Parse policies, if they are given
        try:
            policy = policy_text and json.loads(policy_text)
        except ValueError:
            result["policy"] = policy_text
            result["msg"] = "Could not parse policy"
            return False, result

        try:
            lifecycle_policy = lifecycle_policy_text and json.loads(lifecycle_policy_text)
        except ValueError:
            result["lifecycle_policy"] = lifecycle_policy_text
            result["msg"] = "Could not parse lifecycle_policy"
            return False, result

        result["state"] = state
        result["created"] = False

        repo = ecr.get_repository(registry_id, name)

        if state == "present":
            result["created"] = False

            if not repo:
                repo = ecr.create_repository(registry_id, name, image_tag_mutability, encryption_configuration)
                result["changed"] = True
                result["created"] = True
            else:
                if encryption_configuration is not None:
                    if repo.get("encryptionConfiguration") != encryption_configuration:
                        result["msg"] = "Cannot modify repository encryption type"
                        return False, result

                repo = ecr.put_image_tag_mutability(registry_id, name, image_tag_mutability)
            result["repository"] = repo

            if purge_lifecycle_policy:
                original_lifecycle_policy = ecr.get_lifecycle_policy(registry_id, name)

                result["lifecycle_policy"] = None

                if original_lifecycle_policy:
                    ecr.purge_lifecycle_policy(registry_id, name)
                    result["changed"] = True

            elif lifecycle_policy_text is not None:
                try:
                    result["lifecycle_policy"] = lifecycle_policy
                    original_lifecycle_policy = ecr.get_lifecycle_policy(registry_id, name)

                    if compare_policies(original_lifecycle_policy, lifecycle_policy):
                        ecr.put_lifecycle_policy(registry_id, name, lifecycle_policy_text)
                        result["changed"] = True
                except Exception:
                    # Some failure w/ the policy. It's helpful to know what the
                    # policy is.
                    result["lifecycle_policy"] = lifecycle_policy_text
                    raise

            if purge_policy:
                original_policy = ecr.get_repository_policy(registry_id, name)

                result["policy"] = None

                if original_policy:
                    ecr.delete_repository_policy(registry_id, name)
                    result["changed"] = True

            elif policy_text is not None:
                try:
                    # Sort any lists containing only string types
                    policy = sort_lists_of_strings(policy)

                    result["policy"] = policy

                    original_policy = ecr.get_repository_policy(registry_id, name)
                    if original_policy:
                        original_policy = sort_lists_of_strings(original_policy)

                    if compare_policies(original_policy, policy):
                        ecr.set_repository_policy(registry_id, name, policy_text, force_set_policy)
                        result["changed"] = True
                except Exception:
                    # Some failure w/ the policy. It's helpful to know what the
                    # policy is.
                    result["policy"] = policy_text
                    raise

            else:
                original_policy = ecr.get_repository_policy(registry_id, name)
                if original_policy:
                    result["policy"] = original_policy

            original_scan_on_push = ecr.get_repository(registry_id, name)
            if original_scan_on_push is not None:
                if scan_on_push != original_scan_on_push["imageScanningConfiguration"]["scanOnPush"]:
                    result["changed"] = True
                    result["repository"]["imageScanningConfiguration"]["scanOnPush"] = scan_on_push
                    ecr.put_image_scanning_configuration(registry_id, name, scan_on_push)

        elif state == "absent":
            result["name"] = name
            if repo:
                ecr.delete_repository(registry_id, name, force_absent)
                result["changed"] = True

    except Exception as err:
        msg = str(err)
        if isinstance(err, botocore.exceptions.ClientError):
            msg = boto_exception(err)
        result["msg"] = msg
        result["exception"] = traceback.format_exc()
        return False, result

    if ecr.skipped:
        result["skipped"] = True

    if ecr.changed:
        result["changed"] = True

    return True, result


def main():
    argument_spec = dict(
        name=dict(required=True),
        registry_id=dict(required=False),
        state=dict(required=False, choices=["present", "absent"], default="present"),
        force_absent=dict(required=False, type="bool", default=False),
        force_set_policy=dict(required=False, type="bool", default=False),
        policy=dict(required=False, type="json"),
        image_tag_mutability=dict(required=False, choices=["mutable", "immutable"], default="mutable"),
        purge_policy=dict(required=False, type="bool"),
        lifecycle_policy=dict(required=False, type="json"),
        purge_lifecycle_policy=dict(required=False, type="bool"),
        scan_on_push=(dict(required=False, type="bool", default=False)),
        encryption_configuration=dict(
            required=False,
            type="dict",
            options=dict(
                encryption_type=dict(required=False, type="str", default="AES256", choices=["AES256", "KMS"]),
                kms_key=dict(required=False, type="str", no_log=False),
            ),
            required_if=[
                ["encryption_type", "KMS", ["kms_key"]],
            ],
        ),
    )
    mutually_exclusive = [
        ["policy", "purge_policy"],
        ["lifecycle_policy", "purge_lifecycle_policy"],
    ]

    module = AnsibleAWSModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        mutually_exclusive=mutually_exclusive,
    )

    ecr = EcsEcr(module)
    passed, result = run(ecr, module.params)

    if passed:
        module.exit_json(**result)
    else:
        module.fail_json(**result)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2024, Sabari Jaganathan (@sajagana) <sajagana@cisco.com>

# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {"metadata_version": "1.1", "status": ["preview"], "supported_by": "community"}

DOCUMENTATION = r"""
---
module: ndo_l3out_annotation
short_description: Manage L3Outs Annotation on Cisco Nexus Dashboard Orchestrator (NDO).
description:
- Manage L3Outs Annotation on Cisco Nexus Dashboard Orchestrator (NDO).
- This module is only supported on ND v3.1 (NDO v4.3) and later.
author:
- Sabari Jaganathan (@sajagana)
options:
  template:
    description:
    - The name of the template.
    - The template must be a L3Out template.
    type: str
    aliases: [ l3out_template ]
    required: true
  l3out:
    description:
    - The name of the L3Out.
    type: str
    aliases: [ l3out_name ]
    required: true
  annotation_key:
    description:
    - The key of the Annotation object.
    type: str
    aliases: [ key ]
  annotation_value:
    description:
    - The value of the Annotation object.
    type: str
    aliases: [ value ]
  state:
    description:
    - Use C(absent) for removing.
    - Use C(query) for listing an object or multiple objects.
    - Use C(present) for creating or updating.
    type: str
    choices: [ absent, query, present ]
    default: query
extends_documentation_fragment: cisco.mso.modules
notes:
- The O(template) must exist before using this module in your playbook.
  Use M(cisco.mso.ndo_template) to create the L3Out template.
- The O(l3out) must exist before using this module in your playbook.
  Use M(cisco.mso.ndo_l3out_template) to create the L3Out.
"""

EXAMPLES = r"""
- name: Add an annotation with key and value
  cisco.mso.ndo_l3out_annotation:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_l3out_template
    l3out: "L3OutAnnotation"
    annotation_key: "annotation_key_1"
    annotation_value: "annotation_value_1"
    state: present

- name: Update an annotation with value
  cisco.mso.ndo_l3out_annotation:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_l3out_template
    l3out: "L3OutAnnotation"
    annotation_key: "annotation_key_1_updated"
    annotation_value: "annotation_value_1"
    state: present

- name: Query a specific annotation with key
  cisco.mso.ndo_l3out_annotation:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_l3out_template
    l3out: "L3OutAnnotation"
    annotation_key: "annotation_key_1_updated"
    state: query
  register: query_one

- name: Query all annotations
  cisco.mso.ndo_l3out_annotation:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_l3out_template
    l3out: "L3OutAnnotation"
    state: query
  register: query_all

- name: Delete an annotation
  cisco.mso.ndo_l3out_annotation:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_l3out_template
    l3out: "L3OutAnnotation"
    annotation_key: "annotation_key_1_updated"
    state: absent
"""

RETURN = r"""
"""


import copy
from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.mso.plugins.module_utils.mso import MSOModule, mso_argument_spec
from ansible_collections.cisco.mso.plugins.module_utils.template import MSOTemplate, KVPair


def get_annotation_object(mso_template_object, l3out_name, annotation_key):
    l3outs = mso_template_object.template.get("l3outTemplate", {}).get("l3outs", [])
    l3out_object = mso_template_object.get_object_by_key_value_pairs("L3Outs", l3outs, [KVPair("name", l3out_name)], True)
    annotations = l3out_object.details.get("tagAnnotations", [])
    if annotation_key:
        annotation = mso_template_object.get_object_by_key_value_pairs("L3Out Annotations", annotations, [KVPair("tagKey", annotation_key)])
        return l3out_object, annotation

    return l3out_object, annotations


def main():
    argument_spec = mso_argument_spec()
    argument_spec.update(
        template=dict(type="str", required=True, aliases=["l3out_template"]),
        l3out=dict(type="str", required=True, aliases=["l3out_name"]),
        annotation_key=dict(type="str", aliases=["key"], no_log=False),
        annotation_value=dict(type="str", aliases=["value"]),
        state=dict(type="str", default="query", choices=["absent", "query", "present"]),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ["state", "absent", ["annotation_key"]],
            ["state", "present", ["annotation_key", "annotation_value"]],
        ],
    )

    mso = MSOModule(module)

    template = module.params.get("template")
    l3out = module.params.get("l3out")
    annotation_key = module.params.get("annotation_key")
    annotation_value = module.params.get("annotation_value")
    state = module.params.get("state")

    l3out_template_object = MSOTemplate(mso, "l3out", template)
    l3out_template_object.validate_template("l3out")

    l3out_object, annotation = get_annotation_object(l3out_template_object, l3out, annotation_key)

    if annotation_key:
        if annotation is not None:
            mso.existing = mso.previous = copy.deepcopy(annotation.details)  # Query a specific Annotation
    else:
        mso.existing = annotation  # Query all

    path = "/l3outTemplate/l3outs/{0}/tagAnnotations".format(l3out_object.index)

    ops = []
    if state == "present":
        if mso.existing:
            proposed_payload = copy.deepcopy(mso.existing)
            if annotation_value and proposed_payload.get("tagValue") != annotation_value:
                ops.append(dict(op="replace", path="{0}/{1}/tagValue".format(path, annotation.index), value=annotation_value))
                proposed_payload["tagValue"] = annotation_value
            mso.sanitize(proposed_payload)
        else:
            payload = {"tagKey": annotation_key, "tagValue": annotation_value}
            ops.append(dict(op="add", path="{0}/-".format(path), value=copy.deepcopy(payload)))
            mso.sanitize(payload)
    elif state == "absent":
        if mso.existing:
            ops.append(dict(op="remove", path="{0}/{1}".format(path, annotation.index)))

    if not module.check_mode and ops:
        l3out_template_object.template = mso.request(l3out_template_object.template_path, method="PATCH", data=ops)
        l3out_object, annotation = get_annotation_object(l3out_template_object, l3out, annotation_key)

        if annotation:
            mso.existing = annotation.details  # When the state is present
        else:
            mso.existing = {}  # When the state is absent
    elif module.check_mode and state != "query":  # When the state is present/absent with check mode
        mso.existing = mso.proposed if state == "present" else {}

    mso.exit_json()


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: networks_sm_trusted_access_configs_info
short_description: Information module for networks _sm _trusted _access _configs
description:
  - Get all networks _sm _trusted _access _configs.
  - List Trusted Access Configs.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module_info
  - cisco.meraki.module_info_pagination
author: Francisco Munoz (@fmunoz)
options:
  headers:
    description: Additional headers.
    type: dict
  networkId:
    description:
      - NetworkId path parameter. Network ID.
    type: str
  perPage:
    description:
      - PerPage query parameter. The number of entries per page returned. Acceptable
        range is 3 - 1000. Default is 100.
    type: int
  startingAfter:
    description:
      - >
        StartingAfter query parameter. A token used by the server to indicate the
        start of the page. Often this is a timestamp or an ID but it is not limited
        to those. This parameter should not be defined by client applications. The
        link for the first, last, prev, or next page in the HTTP Link header should
        define it.
    type: str
  endingBefore:
    description:
      - >
        EndingBefore query parameter. A token used by the server to indicate the end
        of the page. Often this is a timestamp or an ID but it is not limited to those.
        This parameter should not be defined by client applications. The link for
        the first, last, prev, or next page in the HTTP Link header should define
        it.
    type: str
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for sm getNetworkSmTrustedAccessConfigs
    description: Complete reference of the getNetworkSmTrustedAccessConfigs API.
    link: https://developer.cisco.com/meraki/api-v1/#!get-network-sm-trusted-access-configs
notes:
  - SDK Method used are
    sm.Sm.get_network_sm_trusted_access_configs,
  - Paths used are
    get /networks/{networkId}/sm/trustedAccessConfigs,
"""

EXAMPLES = r"""
- name: Get all networks _sm _trusted _access _configs
  cisco.meraki.networks_sm_trusted_access_configs_info:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    perPage: 0
    startingAfter: string
    endingBefore: string
    networkId: string
    total_pages: -1
    direction: next
  register: result
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: list
  elements: dict
  sample: >
    [
      {
        "id": "string",
        "ssidName": "string",
        "name": "string",
        "scope": "string",
        "tags": [
          "string"
        ],
        "timeboundType": "string",
        "sendExpirationEmails": true,
        "notifyTimeBeforeAccessEnds": 0,
        "additionalEmailText": "string",
        "accessStartAt": "string",
        "accessEndAt": "string"
      }
    ]
"""
